#include "../../framework/framework.hpp"
#include "../../framework/output streams.hpp"
#include "../../framework/Memory.hpp"
#include "../../framework/Text.hpp"
#include "../../framework/OS Win32.hpp"

#include "../../framework/SIMD.hpp"
#include "../../framework/GFX.hpp"
#include "../../framework/Formats.hpp"
#include "../../framework/Namco.hpp"
#include "../../framework/Namco AceCombat2.hpp"
#include "../../framework/Sony PSX TIM.hpp"

#include "../../framework/OS NT.hpp"
#include "../../framework/OS NT files.hpp"



// TODO: Supervisor!



void extractWhateverItIs(
	Byte const             data[],
	UInt4B                 dataSize,
	OS::NT::Directory &    itsDirectory,
	NativeCodeUnit const * itsName
);

bool INLINE_CODE tryToExtractDirectory(
	Byte const              directory[],
	UInt4B const            directorySize,
	OS::NT::Directory &     itsDirectory,
	UTF16Unit const * const itsName
) {
	if(no == Namco::isDirectory(directory, directorySize)) {
		return no; // not a directory
	}

	auto const & header        = *reinterpret_cast<Namco::Directory const *>(directory);
	auto const   numberOfFiles = littleEndian(header.numberOfFiles);
	auto         subDirectory  = itsDirectory.tryToOpenOrCreateDirectory(itsName, OS::NT::ObjectSharing::dontCare, nullptr);
	for(auto i = 0u; i < numberOfFiles; ++i) {

		auto const fileOffset = littleEndian(header.fileOffsets[i]);

		// For now, the only way to find out the payloads size is to check the beginning of the payload right after that.
		//   entries are NOT always sorted by ascending offset, so we need to check the entire array
		//   assume file end for worst case
		UInt4B fileSize;
		{
			auto nextFileOffset = directorySize;
			for(auto j = 0u; j < numberOfFiles; ++j) {
				auto const candidate = littleEndian(header.fileOffsets[j]);
				if(fileOffset < candidate && candidate < nextFileOffset) {
					nextFileOffset = candidate;
				}
			}
			fileSize = nextFileOffset - fileOffset;
		}

		NativeCodeUnit name[256];
		*Text::toDecimalString(name, i) = 0;
		extractWhateverItIs(directory + fileOffset, fileSize, *subDirectory, name);
	}
	subDirectory->close();

	return yes;
}

void INLINE_CODE extractDATBlock(
	Byte const                   block[],
	UIntLeast4B const            blockSize,
	OS::NT::Directory &          itsDirectory,
	NativeCodeUnit const * const itsName
) {
	// Parse the header. If it matches our logic, the block contains many files & directories. Otherwise, extract it blindly.
	auto &     header          = *reinterpret_cast<Namco::AceCombat2::DAT_BlockHeader const *>(block);
	auto const size            = littleEndian(header.size);
	auto const numberOfOffsets = littleEndian(header.numberOfOffsets);
	auto const firstOffset     = littleEndian(header.offsets[0]);
	if(size == blockSize && 0 < numberOfOffsets && firstOffset == sizeof header + 4 * numberOfOffsets) { // Archive?

		auto const subDirectory = itsDirectory.tryToOpenOrCreateDirectory(itsName, OS::NT::ObjectSharing::dontCare, nullptr);
		for(auto i = 0u; i < numberOfOffsets; ++i) {
			auto const fileOffset     = littleEndian(header.offsets[i]);
			auto const nextFileOffset = ((i == numberOfOffsets - 1) ? size : littleEndian(header.offsets[i + 1]));
			auto const fileSize       = nextFileOffset - fileOffset;

			if(fileOffset + fileSize > size) {
				continue; // offset out-of-bounds
			}

			NativeCodeUnit childName[32];
			*Text::toDecimalString(childName, i) = 0;
			extractWhateverItIs(block + fileOffset, fileSize, *subDirectory, childName);
		}
		subDirectory->close();

	} else {

		extractWhateverItIs(block, blockSize, itsDirectory, itsName);

	}
}

void extractWhateverItIs(
	Byte const                   data[],
	UInt4B const                 dataSize,
	OS::NT::Directory &          itsDirectory,
	NativeCodeUnit const * const itsName
) {
	if(no == tryToExtractDirectory(data, dataSize, itsDirectory, itsName)) {

		// Save as raw file:
		NativeCodeUnit name[32];
		auto e = Text::copyUntilZero(name, itsName);
		e[0] = '.';
		if(12 < dataSize && Sony::PSX::TIM::isTIM(data, dataSize)) {
			e[1] = 't';
			e[2] = 'i';
			e[3] = 'm';
		} else {
			e[1] = 'd';
			e[2] = 'a';
			e[3] = 't';
		}
		e[4] = 0;

		if(auto f = itsDirectory.tryToCreateEmptyFile(name, OS::NT::FileAccess::writeOnly, OS::NT::ObjectSharing::readOnly, OS::NT::FileCaching::sequential, nullptr)) {
			f->tryToWrite(0, data, dataSize, nullptr);
			f->close();
		}

	}
}



int main() {
	out << "\n"
	       "\t\t\t      U N P A C K _ A C 2\n"
	       "\n"
	       "extracts the data files of Namco's ACE COMBAT 2\n"
	       "\n"
	       "place in one directory with ACE2.DAT\n"
	       "\n"
	       "by Krishty 2017 - contact me on zfx.info\n";

	// Assume Ace Combat 2s data is located in the Current Directory:
	UTF16Unit rootDirectoryPath[4 + 260];
	rootDirectoryPath[0] = '\\';
	rootDirectoryPath[1] = '?';
	rootDirectoryPath[2] = '?';
	rootDirectoryPath[3] = '\\';
	OS::Win32::currentDirectory(rootDirectoryPath + 4);
	if(auto const toCurrentDirectory = tryToOpenDirectory(rootDirectoryPath, OS::NT::ObjectSharing::readOnly, nullptr)) {

		// Map the archive to memory:
		if(auto const toDATFile = toCurrentDirectory->tryToOpenFile(UTF16("ACE2.DAT"), OS::NT::FileAccess::readOnly, OS::NT::ObjectSharing::dontCare, OS::NT::FileCaching::sequential, nullptr)) {
			auto const dat_size = toDATFile->size();
			if(auto const dat_begin = tryToMapForReading(*toDATFile, nullptr)) {

				// Parse the DAT header and extract each block:
				auto &     header           = *reinterpret_cast<Namco::AceCombat2::DAT_Directory const *>(dat_begin);
				auto const extractDirectory = toCurrentDirectory->tryToOpenOrCreateDirectory(UTF16("data"), OS::NT::ObjectSharing::readOnly, nullptr);
				auto const numberOfBlocks   = littleEndian(header.numberOfBlocks);
				for(auto i = 0u; i < numberOfBlocks; ++i) {
					auto & pointer = header.pointers[i];

					NativeCodeUnit name[32];
					*Text::toDecimalString(name, i) = 0;
					extractDATBlock(dat_begin + offsetOf(pointer), sizeOf(pointer), *extractDirectory, name);

				} // for each offset

			} // if DAT mapped
		} // if DAT opened

	} // if Current Directory opened

	return 0;
}