#include "../../framework/framework.hpp"
#include "../../framework/output streams.hpp"
#include "../../framework/Memory.hpp"
#include "../../framework/Text.hpp"
#include "../../framework/OS Win32.hpp"

#include "../../framework/SIMD.hpp"
#include "../../framework/GFX.hpp"
#include "../../framework/Formats.hpp"
#include "../../framework/Sony PSX TIM.hpp"
#include "../../framework/Namco.hpp"
#include "../../framework/Namco AceCombat3.hpp"
#include "../../framework/Namco ULZ.hpp"

#include "../../framework/OS NT.hpp"
#include "../../framework/OS NT files.hpp"
#include "../../framework/OS NT Heap.hpp"



// TODO: Supervisor!



void extractWhateverItIs(
	Byte const           [],
	UInt4B               ,
	OS::NT::Directory &  ,
	NativeCodeUnit const itsName[],
	OS::NT::Heap &
);

bool INLINE_CODE tryToExtractDirectory(
	Byte const           directory[],
	UInt4B const         directorySize,
	OS::NT::Directory &  itsDirectory,
	NativeCodeUnit const itsName[],
	OS::NT::Heap &       heap
) {
	if(no == Namco::isDirectory(directory, directorySize)) {
		return no; // not a directory
	}

	auto const & header        = *reinterpret_cast<Namco::Directory const *>(directory);
	auto const   numberOfFiles = littleEndian(header.numberOfFiles);
	auto const   subDirectory  = itsDirectory.tryToOpenOrCreateDirectory(itsName, OS::NT::ObjectSharing::dontCare, nullptr);
	for(auto i = 0u; i < numberOfFiles; ++i) {

		auto const fileOffset = littleEndian(header.fileOffsets[i]);

		// For now, the only way to find out the payloads size is to check the beginning of the payload right after that.
		//   entries are NOT always sorted by ascending offset, so we need to check the entire array
		//   assume file end for worst case
		UInt4B fileSize;
		{
			auto nextFileOffset = directorySize;
			for(auto j = 0u; j < numberOfFiles; ++j) {
				auto const candidate = littleEndian(header.fileOffsets[j]);
				if(fileOffset < candidate && candidate < nextFileOffset) {
					nextFileOffset = candidate;
				}
			}
			fileSize = nextFileOffset - fileOffset;
		}

		NativeCodeUnit name[256];
		*Text::toDecimalString(name, i) = 0;
		extractWhateverItIs(directory + fileOffset, fileSize, *subDirectory, name, heap);
	}
	subDirectory->close();

	return yes;
}

void extractWhateverItIs(
	Byte const           data[],
	UInt4B const         dataSize,
	OS::NT::Directory &  itsDirectory,
	NativeCodeUnit const itsName[],
	OS::NT::Heap &       heap
) {
	// ULZ\x1A denotes an ULZ-compressed file.
	if(Namco::ULZ::isULZ(data, dataSize)) {

		auto const & header        = *reinterpret_cast<Namco::ULZ::Header const *>(data);
		auto const   extractedSize = extractedSizeOf(header);
		if(auto const extracted = heap.tryToReserve(extractedSize)) {

			if(Namco::ULZ::extract(extracted, data, dataSize)) {
				extractWhateverItIs(extracted, extractedSize, itsDirectory, itsName, heap);
			} else {
				INVOKE_DEBUGGER;
			}

			heap.release(extracted);
		} else {
			INVOKE_DEBUGGER;
		}
	
	} else if(tryToExtractDirectory(data, dataSize, itsDirectory, itsName, heap)) {

		// just extracted it; nothing more to do

	} else {

		// Save as raw file:
		NativeCodeUnit name[32];
		auto e = Text::copyUntilZero(name, itsName);
		e[0] = '.';
		if(12 < dataSize && Sony::PSX::TIM::isTIM(data, dataSize)) {
			e[1] = 't';
			e[2] = 'i';
			e[3] = 'm';
		} else {
			e[1] = 'd';
			e[2] = 'a';
			e[3] = 't';
		}
		e[4] = 0;

		if(auto f = itsDirectory.tryToCreateEmptyFile(name, OS::NT::FileAccess::writeOnly, OS::NT::ObjectSharing::readOnly, OS::NT::FileCaching::sequential, nullptr)) {
			f->tryToWrite(0, data, dataSize, nullptr);
			f->close();
		}

	}
}

int main() {
	out << "\n"
	       "\t\t\t      U N P A C K _ A C 3\n"
	       "\n"
	       "extracts the data files of Namco's ACE COMBAT 3: Electrosphere\n"
	       "\n"
	       "place in one directory with ACE.BPB & ACE.BPH\n"
	       "\n"
	       "by Krishty 2017 - contact me on zfx.info\n";

	// All warnings & errors go here:
	Supervisor const * const ui = nullptr;

	// Assume Ace Combat 3s data is located in the Current Directory:
	UTF16Unit rootDirectoryPath[4 + 260 + 1];
	rootDirectoryPath[0] = '\\';
	rootDirectoryPath[1] = '?';
	rootDirectoryPath[2] = '?';
	rootDirectoryPath[3] = '\\';
	OS::Win32::currentDirectory(rootDirectoryPath + 4);
	if(auto const toCurrentDirectory = tryToOpenDirectory(rootDirectoryPath, OS::NT::ObjectSharing::readOnly, ui)) {
		auto & heap = *OS::NT::processEnvironmentBlock().ProcessHeap;

		// Map the data header to memory:
		if(auto const bph = toCurrentDirectory->tryToOpenFile(UTF16("ACE.BPH"), OS::NT::FileAccess::readOnly, OS::NT::ObjectSharing::dontCare, OS::NT::FileCaching::sequential, ui)) {
			auto const bph_size = bph->size();
			if(auto const bph_begin = tryToMapForReading(*bph, ui)) {

				// Map the data to memory:
				if(auto const bpb = toCurrentDirectory->tryToOpenFile(UTF16("ACE.BPB"), OS::NT::FileAccess::readOnly, OS::NT::ObjectSharing::dontCare, OS::NT::FileCaching::sequential, ui)) {
					auto const bpb_size = bpb->size();
					if(auto const bpb_begin = tryToMapForReading(*bpb, ui)) {

						// Extracted files go here:
						auto const   extractDirectory = toCurrentDirectory->tryToOpenOrCreateDirectory(UTF16("data"), OS::NT::ObjectSharing::readOnly, ui);
						// Extract the data:
						auto const & header           = *reinterpret_cast<Namco::AceCombat3::BPH_Header const *>(bph_begin);
						auto const   numberOfPointers = littleEndian(header.numberOfPointers);
						for(auto i = 0u; i < numberOfPointers; ++i) {
							auto & pointer = header.pointers[i];

							// Generate names: 0, 1, 2, 
							NativeCodeUnit name[32];
							*Text::toDecimalString(name, i) = 0;
							extractWhateverItIs(bpb_begin + offsetOf(pointer), sizeOf(pointer), *extractDirectory, name, heap);

						} // for each offset

					} // if BPB mapped
				} // if BPB opened

			} // if BPH mapped
		} // if BPH opened

	} // if Current Directory opened

	return 0;
}