// Namco's data compression method in Ace Combat 3, Ace Combat 5, and Time Crisis.
//
//  ULZ was developed by Junichi Sakai
//    the Japanese Time Crisis executable contains the string ulz ver.1.00 J.Sakai
//    the Japanese Ace Combat 3 executable contains the string ulz decoder ver.1.05 programmed by Junichi Sakai [1]
//
//  archives begin with Ulz (55 6C 7A 1A hexadecimal, or 1A7A6C55 little endian)
//
//  ULZ is LZR (LempelZivRenau)
//    LZSS (LempelZivStorerSzymanski) with separate data streams
//    optimized for MIPS (32-bit words, few variable bit shifts)
//
//  Timeline:
//   1995:        Namco releases Time Crisis (ULZ 1.00)
//   1999:        Namco releases Ace Combat 3 (ULZ 1.05)
//   2004:        Namco released Ace Combat 5 (ULZ 1.05)
//   2006-04-22:  McCuao posts an incomplete description of ULZ from a friend [2]
//   2012-03-11:  Gipphe analyzes the format [3], but gives up  his texts and code is lost
//   2012-06-20:  CUE posts incomplete decompression code for Time Crisis and Ace Combat 3 [4]
//   2014-02-08:  esperknight posts incomplete decompression code for Ace Combat 3 [5]
//   2014-08-02:  esperknight posts full decompression code [6]
//   2017-01-10:  Infrid unites the two separate compression methods into one [7]
//
// [1] [asm Psx] Non Trovo La Routine, Romhacking.it, http://romhacking.it/forums/index.php?topic=605.msg5294
// [2] Time Crisis 1 [PSOne] .ulz (LZR?), XeNTaX, http://forum.xentax.com/viewtopic.php?f=21&t=5686&view=next
// [3] AC3 Data Hacking Progress, USEA Today, http://useatoday.blogspot.de/2012/03/ac3-data-hacking-progress.html
// [4] [PSX] Ace Combat 3 - I'm looking for help, Romxhacking, http://romxhack.esforos.com/post2916
// [5] [PS1]ACE COMBAT 3: text replacement (Jpn.->Eng.), ROMhacking.net, http://www.romhacking.net/forum/index.php/topic,17658.msg256925.html#msg256925
// [6] [PS1]ACE COMBAT 3: text replacement (Jpn.->Eng.), ROMhacking.net, http://www.romhacking.net/forum/index.php/topic,17658.msg264383.html#msg264383
// [7] [Projekt] Extracting Ace Combat, ZFX.info, https://zfx.info/viewtopic.php?f=10&t=4110&p=53943#p53827



namespace Namco {


	// Namco's data compression method in Ace Combat 3, Ace Combat 5, and Time Crisis.
	namespace ULZ {


		struct Header {
			// Always Ulz\x1A (55 6C 7A 1A hexadecimal).
			UInt4B_LE id; 
			// bits  023: extracted size
			// bits 2431: compression type (0 or 2)
			UInt4B_LE typeAndSize;
			// bits  023: offset of symbol table from beginning of archive
			// bits 2431: log2 of sliding window
			UInt4B_LE u_pos;
			// Offset of lookups from beginning of archive.
			UInt4B_LE c_pos;
			UInt4B_LE flags[];
		};


		// Checks whether the given bytes mark the beginning of an ULZ stream.
		//   pass at least 20 bytes
		bool isULZ(Byte const [], USize);

		// Returns the size of the extracted data. ULZ supports file sizes up to 24 MiB.
		UInt4B extractedSizeOf(Header const &);

		// Extracts the given ULZ to the given address.
		//   returns no on failure
		//   requires sufficient space  use extractedSizeOf()
		bool extract(
			Byte *       toOutput,
			Byte const * toInput,
			USize        inputSize
		);


	} // ULZ


} // Namco