// Data structures in games by Namco, a Japanese video game developer.



#include "framework.hpp"
#include "Namco.hpp"



namespace Namco {


	// Checks whether the given blob can be treated as a directory.
	bool isDirectory(Byte const data[], USize const size) {
		if(size < sizeof(Directory)) {
			return no;
		}

		auto const & header        = *reinterpret_cast<Directory const *>(data);
		auto const   numberOfFiles = littleEndian(header.numberOfFiles);
		auto const   headerSize    = sizeof header + sizeof(UInt4B_LE) * numberOfFiles;
		if(0 == numberOfFiles || size < headerSize) {
			return no; // too short for all entries
		}
		// Enhanced checks are necessary to handle Air Combat properly:
		//   files must start after the header and before the file end
		for(auto i = 0u; i < numberOfFiles; ++i) {
			auto const offset = littleEndian(header.fileOffsets[i]);
			if(offset < headerSize || offset >= size) {
				return no;
			}
		}

		return yes;
	}


	UIntLeast4B offsetOf(CDPointer const & pointer) {
		return littleEndian(pointer.offsetBy2048) * 2048;
	}

	UIntLeast4B sizeOf(CDPointer const & pointer) {
		return 4 * (littleEndian(pointer.flagsAndSizeBy4) & 0x0FFF'FFFF);
	}


} // Namco