// Data structures in games by Namco, a Japanese video game developer.



// Namco is a Japanese video game developer. Anything concerning its games is bundled here.
namespace Namco {


	// A blob with an arbitrary number of sub-blobs in it. Probably represents a directory.
	//   packed game data, much like ids WAD or an uncompressed ZIP archive
	//   used by
	//     Air Combat (all .MD and .TD files)
	//     Ace Combat 2
	//     Ace Combat 3
	//     Ace Combat 5
	struct Directory {
		UInt4B_LE numberOfFiles;
		// Each files offset in the blob.
		//   relative to beginning of blob
		//     nth offset cannot be lower than 4 + 4  n B
		//     Aligned to 4-B boundaries?
		//   usually sorted, but not always
		//     mess in some Ace Combat 2 directories
		//   file size is the distance to the next (unsorted!) offset or to the end of the blob
		UInt4B_LE fileOffsets[];
	};

	// Checks whether the given blob can be treated as a directory.
	bool isDirectory(Byte const [], USize);


	// Points to a data block which can be read directly from the CD-ROM.
	//   always aligned to 2 KiB (CD-ROM sector size)
	//   used by
	//     Ace Combat 2
	//     Ace Combat 3
	struct CDPointer {
		//  8 MSB: unknown flags
		// 24 LSB: size divided by four
		UInt4B_LE flagsAndSizeBy4;
		UInt4B_LE offsetBy2048;
	};

	UIntLeast4B offsetOf(CDPointer const &);

	UIntLeast4B sizeOf(CDPointer const &);


} // Namco