// Helpers for Windows NT heap management.


#include "framework.hpp"
#include "OS NT.hpp"
#include "OS NT Heap.hpp"



import_from(ntdll) {

	__declspec(restrict) OS::NT::Heap * API_CALL RtlCreateHeap(
		UInt4B Flags,
		void * HeapBase,
		USize  ReserveSize,
		USize  CommitSize,
		void * Lock,
		void * Parameters
	);

	__declspec(restrict) Byte * API_CALL RtlAllocateHeap(
		OS::NT::Heap *,
		UInt4B        Flags,
		USize
	);

	__declspec(restrict) Byte * API_CALL RtlReAllocateHeap( // undocumented :(
		OS::NT::Heap *,
		UInt4B        Flags,
		void const *  ,
		USize
	);

	TODO("MSDN says the return type is LOGICAL, but I can't find a definition for that")
	int API_CALL RtlFreeHeap(
		OS::NT::Heap *,
		UInt4B        Flags,
		void const *
	);

	OS::NT::Heap * API_CALL RtlDestroyHeap(OS::NT::Heap *);


} // ntdll.dll



namespace OS {


	namespace NT {


		// Reserves memory of the given size, in bytes.
		//   content is undefined
		//   must be release()d (even with zero size)
		//   returns nullptr on failure
		Byte * Heap::tryToReserve(USize const size) {
			return RtlAllocateHeap(this, 0, size);
		}

		// Reserves memory of the given size, in bytes.
		//   content is zeroed
		//   must be release()d (even with zero size)
		//   returns nullptr on failure
		Byte * Heap::tryToReserveZeroed(USize const size) {
			return RtlAllocateHeap(this, 8, size); // HEAP_ZERO_MEMORY
		}

		// Resizes the given memory block to the given size.
		//   input must not be nullptr
		//   new content is undefined
		//   must be release()d (even with zero size)
		//   on failure, returns nullptr and leaves input untouched
		Byte * Heap::tryToReallocate(
			void const * oldAddress,
			USize const  newSize
		) {
			return RtlReAllocateHeap(this, 0, oldAddress, newSize);
		}

		void Heap::release(void const * memory) {
			auto const result = RtlFreeHeap(this, 0, memory);
			MUST(0 != result); // Heap corruption?
		}

		// Destroys the heap and all memory in it.
		void Heap::destroy() {
			auto const result = RtlDestroyHeap(this);
			MUST(nullptr == result); // Heap corruption?
		}

		//  must be destroy()d
		//  returns nullptr on failure
		Heap * tryToCreateHeap() {
			return RtlCreateHeap(
				2, nullptr, // HEAP_GROWABLE
				0, 0,       // nothing reserved or committed
				nullptr,    // no custom lock
				nullptr     // no custom parameters
			);
		}


	} // NT


} // OS