// Helpers for Windows NT heap management.



namespace OS {


	namespace NT {


		// PVOID in NTs heap functions.
		struct Heap : Opaque {

			// Reserves memory of the given size, in bytes.
			//   content is undefined
			//   must be release()d (even with zero size)
			//   returns nullptr on failure
			Byte * tryToReserve(USize);

			// Reserves memory of the given size, in bytes.
			//   content is zeroed
			//   must be release()d (even with zero size)
			//   returns nullptr on failure
			Byte * tryToReserveZeroed(USize);

			// Resizes the given memory block to the given size.
			//   input must not be nullptr
			//   new content is undefined
			//   must be release()d (even with zero size)
			//   on failure, returns nullptr and leaves input untouched
			Byte * tryToReallocate(void const *, USize);

			void release(void const *);

			// Destroys the heap and all memory in it.
			void destroy();

		};

		//  must be destroy()d
		//  returns nullptr on failure
		Heap * tryToCreateHeap();


	} // NT


} // OS