// Helpers for Windows NT functionality (files, processes, threads, modules, ).


#include "framework.hpp"
#include "OS NT.hpp"
#include "errors/OS Win32.hpp" // TODO: make own error codes



extern "C" {
	unsigned long __readfsdword(unsigned long Offset);
	unsigned __int64 __readgsqword(unsigned long Offset);
}

import_from(ntdll) {

	OS::NT::Status API_CALL NtTerminateProcess(
		OS::NT::Process *,
		OS::NT::Status   ExitStatus
	);

} // ntdll.dll



namespace OS {


	namespace NT {


		// Breaks into the debugger if the given status is not successful.
		void mustSucceed(Status const status) {
#			if _DEBUG
				if(0 > status) {
					INVOKE_DEBUGGER;
				}
#			else
				UNUSED(status);
#			endif
		}

		// Reports the given error to the supervisor.
		void reportError(Supervisor const * const supervisorOrNull, Status const status) {
			report(supervisorOrNull, OS::Win32::Error::unspecified, (void *)USize(status));
		}


		// Terminates the calling process immediately without any cleanup or DLL detaching.
		WONT_RETURN terminateProcess(Status exitCode) {
			mustSucceed(NtTerminateProcess(thisProcess, exitCode));
			UNREACHABLE;
		}

		PEB & processEnvironmentBlock() {
#			if COMPILED_FOR_X86_32
				return *(PEB *)__readfsdword(0x30);
#			elif COMPILED_FOR_X86_64
				return *(PEB *)__readgsqword(0x60);
#			else
#				error TODO
#			endif
		}


	} // NT


} // OS