// Helpers for Windows NT functionality (files, processes, threads, modules, ).



namespace OS {


	namespace NT {


		using Status = SInt4B; // NTSTATUS

		void mustSucceed(Status);

		// Reports the given error to the supervisor.
		void reportError(Supervisor const *, Status);


		// NT handles are always a multiple of four (see The Old New Thing, Why are kernel HANDLEs always a multiple of four?).
		auto constexpr handleAlignment = 4;


		// HANDLE in NTs file and driver functions.
		//   always synchronous
		struct alignas(handleAlignment) File;


		// HANDLE in NTs file functions.
		struct alignas(handleAlignment) Directory;


		// HANDLE in NTs process management functions.
		struct alignas(handleAlignment) Process;


		// HANDLE in NTs thread management functions.
		struct alignas(handleAlignment) Thread;


		// HANDLE in NTs virtual memory section functions.
		struct alignas(handleAlignment) Section;


		// PVOID in NTs heap functions.
		struct Heap;


		// NtCurrentProcess() pseudo-handle:
		auto const thisProcess = reinterpret_cast<Process *>(-1);

		// Terminates the calling process immediately without any cleanup or DLL detaching.
		WONT_RETURN terminateProcess(Status);


		struct LoadedModules; // PEB_LDR_DATA

		struct ProcessParameters; // RTL_USER_PROCESS_PARAMETERS

		struct PEB {
#			if COMPILED_FOR_X86_32
				Byte                Reserved1[2];
				Byte                BeingDebugged;
				Byte                Reserved2[1];
				void *              Reserved3[2];
				LoadedModules *     Ldr;
				ProcessParameters * ProcessParameters;
				void *              SubSystemData;
				Heap *              ProcessHeap;
				Byte                Reserved4[96];
				void *              Reserved5[52];
				void *              PostProcessInitRoutine;
				Byte                Reserved6[128];
				void *              Reserved7[1];
				UInt4B              SessionId;
#			elif COMPILED_FOR_X86_64
				Byte                Reserved1[2];
				Byte                BeingDebugged;
				Byte                Reserved2[21];
				LoadedModules *     Ldr;
				ProcessParameters * ProcessParameters;
				void *              SubSystemData;
				Heap *              ProcessHeap;
				Byte                Reserved3[504];
				void *              PostProcessInitRoutine;
				Byte                Reserved4[136];
				UInt4B              SessionId;
				PAD_4B;
#			else
#				error TODO
#			endif
		} & processEnvironmentBlock();


	} // NT


} // OS