// Helpers for Windows debugging (call stacks, symbol names, ).



namespace OS {


	namespace Win32 {


		namespace DbgHelp {


			struct CallStack {
				void const * callees[8]; // eight callees are sufficient; nullptr for unused slots
			};


			struct SymbolName {
				ASCIICharacter zeroTerminated[256];
			};


			// Links a program symbol to a specific file and line of source code.
			struct SourceLink {
				UTF16Unit const * toZeroTerminatedPath; // owned by the DbgHelp library
				UInt4B            line;
			};


			// Retrieves the current call stack. The upper frames can be skipped; this should be at least one (to ignore the
			//  currentCallStack() call).
			CallStack currentCallStack(UIntLeast1B howManyCalleesToSkip);

			// Returns the name of the symbol at the given address in the modules of the current process.
			SymbolName nameOfSymbolAt(void const *);

			// Returns the source file path and the line number of the symbol at the given address in the current process. If an error
			//  occured, the path is nullptr and the line is zero.
			SourceLink sourceOfSymbolAt(void const *);

			// Prints the given call stack as source file and line number to the debuggers output window. With Visual Studio, the
			//  output can be double-clicked to open the corresponding file.
			void print(CallStack const &);

			// Prints the source file path, line number, and name of the symbol at the given address.
			void printSymbolAt(void const *);


		} // DbgHelp


	} // Win32


} // OS