import_from(Kernel32) {

	Byte * API_CALL VirtualAlloc(
		void * lpAddress,
		USize  dwSize,
		UInt4B flAllocationType,
		UInt4B flProtect
	);

	UInt4B API_CALL VirtualFree(
		void const * lpAddress,
		USize        dwSize,
		UInt4B       dwFreeType
	);

} // Kernel32


locals {

	struct MemoryBlock {
#		if _DEBUG
			Windows::DbgHelp::CallStack callStack;
#		endif
		UInt8B                      size; // must be 8 B for alignment
		UInt8B                      id;   // always 0x13371337; must be 8 B for alignment
		Byte                        data[];
	};

	UInt8B const blockID = 0x1337133713371337;

} // local


namespace Windows {


	local auto const pageSize              = 4096;
	local auto const allocationGranularity = 65536;


	Byte * tryToReserveZeroed(USize size) {
		auto const protectSize = pageSize; // one write-protected page after the payload
		auto const actualSize  = sizeof(MemoryBlock) + size;
		auto const paddingSize = (pageSize - actualSize % pageSize) % pageSize;

		// Allocate the memory (MEM_RESERVE + MEM_COMMIT and PAGE_READWRITE):
		auto result = reinterpret_cast<MemoryBlock *>(VirtualAlloc(nullptr, paddingSize + actualSize + protectSize, 0x00003000, 0x04));
		if(nullptr == result) {
			INVOKE_DEBUGGER;
		}
		result = byteOffset(result, paddingSize);

		// Protect the additional last page (PAGE_NOACCESS):
		UInt4B dummy;
		WINAPI_SHOULD_SUCCEED(VirtualProtect(result->data + size, 1, 0x01, &dummy));

#		if _DEBUG
			result->callStack = DbgHelp::currentCallStack(2);
#		endif
		result->id        = blockID;
		result->size      = size;
		return result->data; // memory from VirtualAlloc() is always zeroed
	}
	Byte * tryToReserve(USize size) {
		auto result = tryToReserveZeroed(size);
		stosb(reinterpret_cast<Byte *>(result), 0xFF, size); // set to FFFFF
		return result;
	}
	Byte * tryToReallocate(void const * previousAddress, USize newSize) {
		auto previousAddressB = reinterpret_cast<Byte const *>(previousAddress);
		auto toBlock          = reinterpret_cast<MemoryBlock const *>(previousAddressB - sizeof(MemoryBlock));
		MUST(blockID == toBlock->id); // Someone allocated wildly?

		auto result = tryToReserve(newSize);
		copy(reinterpret_cast<Byte *>(result), previousAddressB, previousAddressB + minimumOf(USize(toBlock->size), newSize));
		release(previousAddress);

		return result;
	}
	void release(void const * address) {
		auto toBlock = reinterpret_cast<MemoryBlock const *>(reinterpret_cast<Byte const *>(address) - sizeof(MemoryBlock));
		MUST(blockID == toBlock->id); // Someone allocated wildly?

		// The allocation started at the next lower page boundary, although it returned a higher address:
		auto const paddingSize = (pageSize - (sizeof(MemoryBlock) + toBlock->size) % pageSize) % pageSize;
		auto actualAddress = reinterpret_cast<Byte const *>(address) - sizeof(MemoryBlock) - paddingSize;
		WINAPI_SHOULD_SUCCEED(VirtualFree(actualAddress, 0, 0x8000)); // MEM_RELEASE
	}


	void printMemoryLeaks() { }


} // Windows