// Helpers for basic Win32 functionality (files, processes, threads, modules, ).



#include "framework.hpp"
#include "SIMD.hpp"
#include "Text.hpp"
#include "GFX.hpp"
#include "Memory.hpp"
#include "Formats.hpp"
#include "Formats BMP.hpp"
#include "OS Win32.hpp"
#include "errors/OS Win32.hpp"
#if _DEBUG
#	include "OS Win32 DbgHelp.hpp"
#endif

#if COMPILED_BY_VISUAL_CPP
	// Visual C++ intrinsics. On Clang and GCC, the __builtin_ counterparts are used, so no declaration is required.
	import UInt8B __rdtsc();
#endif

import_from(Kernel32) {

	UInt4B API_CALL GetLastError();

	UInt2B API_CALL RtlCaptureStackBackTrace(
		UInt4B   FramesToSkip,
		UInt4B   FramesToCapture,
		void * * BackTrace,
		UInt4B * BackTraceHash
	);

	UInt4B API_CALL FormatMessageW(
		UInt4B       dwFlags,
		void const * lpSource,
		UInt4B       dwMessageId,
		UInt4B       dwLanguageId,
		UTF16Unit *  ,
		UInt4B       nSize,
		void *       Arguments
	);

#	if _DEBUG

		UInt4B API_CALL FormatMessageA(
			UInt4B          dwFlags,
			void const *    lpSource,
			UInt4B          dwMessageId,
			UInt4B          dwLanguageId,
			ASCIICharacter *,
			UInt4B          nSize,
			void *          Arguments
		);

		void API_CALL OutputDebugStringA(ASCIICharacter const *);

#	endif

	UInt4B API_CALL GetModuleHandleExW(
		UInt4B               dwFlags,
		UTF16Unit const *    lpModuleName,
		OS::Win32::Module * *
	);

	UTF16Unit const * API_CALL GetCommandLineW();

	OS::Win32::Process * API_CALL GetCurrentProcess();

	UInt4B API_CALL GetCurrentDirectoryW(UInt4B nBufferLength, UTF16Unit * lpBuffer);

	int API_CALL SetCurrentDirectoryW(UTF16Unit const *);

	WONT_RETURN API_CALL ExitProcess(UInt4B);

	int API_CALL TerminateProcess(OS::Win32::Process *, UInt4B);

	UInt4B API_CALL SetThreadExecutionState(UInt4B);

	UInt4B API_CALL WaitForSingleObject(void const *, UInt4B);

	OS::Win32::File * API_CALL GetStdHandle(SInt4B);

	struct SECURITY_ATTRIBUTES {
		UInt4B nLength;
		PAD_4B_ON_X86_64;
		void * lpSecurityDescriptor;
		int    bInheritHandle;
		PAD_4B_ON_X86_64;
	};

	OS::Win32::File * API_CALL CreateFileW(
		UTF16Unit const *           lpFileName,
		UInt4B                      dwDesiredAccess,
		UInt4B                      dwShareMode,
		SECURITY_ATTRIBUTES const * lpSecurityAttributes,
		UInt4B                      dwCreationDisposition,
		UInt4B                      dwFlagsAndAttributes,
		OS::Win32::File *           hTemplateFile
	);

	struct OVERLAPPED {
		USize  Internal;
		USize  InternalHigh;
		union {
			UInt8B Offset;
			void * Pointer;
		};
		void * hEvent;
	};

	int API_CALL ReadFile(
		OS::Win32::File const *,
		void const *           ,
		UInt4B                 nNumberOfBytesToRead,
		UInt4B *               lpNumberOfBytesRead,
		OVERLAPPED *
	);

	int API_CALL CloseHandle(void const *);

	int API_CALL CreatePipe(
		OS::Win32::File * *         hReadPipe,
		OS::Win32::File * *         hWritePipe,
		SECURITY_ATTRIBUTES const * lpPipeAttributes,
		UInt4B                      nSize
	);

	OS::Win32::Region * API_CALL GetProcessHeap();

	__declspec(restrict) OS::Win32::Region * API_CALL HeapCreate(
		UInt4B flOptions,
		USize  dwInitialSize,
		USize  dwMaximumSize
	);

	__declspec(restrict) Byte * API_CALL HeapAlloc(
		OS::Win32::Region *,
		UInt4B             dwFlags,
		USize              dwBytes
	);

	__declspec(restrict) Byte * API_CALL HeapReAlloc(
		OS::Win32::Region *,
		UInt4B             dwFlags,
		void const *       ,
		USize              dwBytes
	);

	int API_CALL HeapFree(OS::Win32::Region *, UInt4B dwFlags, void const *);

	int API_CALL HeapDestroy(OS::Win32::Region *);

	int API_CALL VirtualProtect(
		void *   ,
		USize    ,
		UInt4B   flNewProtect,
		UInt4B * lpflOldProtect
	);

	int API_CALL VirtualUnlock(void *, USize);

	struct GLOBAL;

	GLOBAL * API_CALL GlobalAlloc(
		UInt4B uFlags,
		USize
	);

	void * API_CALL GlobalLock(GLOBAL *);

	int API_CALL GlobalUnlock(GLOBAL *);

	GLOBAL * API_CALL GlobalFree(GLOBAL *);

	int API_CALL SetProcessWorkingSetSize(
		OS::Win32::Process *,
		USize               dwMinimumWorkingSetSize,
		USize               dwMaximumWorkingSetSize
	);

	struct PROCESS_MEMORY_COUNTERS_EX {
		UInt4B cb;
		UInt4B PageFaultCount;
		USize  PeakWorkingSetSize;
		USize  WorkingSetSize;
		USize  QuotaPeakPagedPoolUsage;
		USize  QuotaPagedPoolUsage;
		USize  QuotaPeakNonPagedPoolUsage;
		USize  QuotaNonPagedPoolUsage;
		USize  PagefileUsage;
		USize  PeakPagefileUsage;
		USize  PrivateUsage;
	};
	int API_CALL K32GetProcessMemoryInfo(
		OS::Win32::Process *        ,
		PROCESS_MEMORY_COUNTERS_EX *,
		UInt4B
	);

	struct MEMORYSTATUSEX {
		UInt4B dwLength;
		UInt4B dwMemoryLoad;
		UInt8B ullTotalPhys;
		UInt8B ullAvailPhys;
		UInt8B ullTotalPageFile;
		UInt8B ullAvailPageFile;
		UInt8B ullTotalVirtual;
		UInt8B ullAvailVirtual;
		UInt8B ullAvailExtendedVirtual;
	};
	int API_CALL GlobalMemoryStatusEx(MEMORYSTATUSEX *);

	int API_CALL SetConsoleScreenBufferSize(OS::Win32::File *, OS::Win32::PointOnConsole);

	int API_CALL GetConsoleMode(OS::Win32::File *, UInt4B *);

	int API_CALL ReadConsoleW(
		OS::Win32::File * hConsoleInput,
		void *            lpBuffer,
		UInt4B            nNumberOfCharsToRead,
		UInt4B *          lpNumberOfCharsRead,
		void *            pInputControl
	);

	int API_CALL QueryPerformanceFrequency(UInt8B *);

	int API_CALL QueryPerformanceCounter(UInt8B *);

	struct SYSTEMTIME {
		UInt2B wYear;
		UInt2B wMonth;
		UInt2B wDayOfWeek;
		UInt2B wDay;
		UInt2B wHour;
		UInt2B wMinute;
		UInt2B wSecond;
		UInt2B wMilliseconds;
	};
	void API_CALL GetSystemTime(SYSTEMTIME *);

	int API_CALL MultiByteToWideChar(
		UInt4B                 CodePage,
		UInt4B                 dwFlags,
		ASCIICharacter const * lpMultiByteStr,
		int                    cbMultiByte,
		UTF16Unit *            lpWideCharStr,
		int                    cchWideChar
	);

	UInt4B API_CALL GetModuleFileNameW(OS::Win32::Module const *, UTF16Unit *, UInt4B nSize);

} // Kernel32

import_from(User32) {

	int API_CALL SetCursorPos(int, int);

	OS::Win32::Icon * API_CALL LoadCursorW(OS::Win32::Module *, UCount);

	OS::Win32::Icon * API_CALL SetCursor(OS::Win32::Icon *);

	int API_CALL OpenClipboard(void * hWndNewOwner);

	int API_CALL EmptyClipboard();

	GLOBAL * API_CALL GetClipboardData(UInt4B uFormat);

	void * API_CALL SetClipboardData(
		UInt4B   uFormat,
		GLOBAL *
	);

	int API_CALL CloseClipboard();

	int API_CALL SystemParametersInfoW(
		UInt4B uiAction,
		UInt4B uiParam,
		void * pvParam,
		UInt4B fWinIni
	);

	int API_CALL GetKeyNameTextW(
		SInt4B      lParam,
		UTF16Unit * ,
		int         cchSize
	);

	UInt4B API_CALL MapVirtualKeyW(
		UInt4B uCode,
		UInt4B uMapType
	);

} // User32

import_from(Shell32) {

	UTF16Unit const * * API_CALL CommandLineToArgvW(UTF16Unit const *, UInt4B * pNumArgs);

} // Shell32


struct IMAGE_DOS_HEADER {
	UInt2B e_magic;
	UInt2B e_cblp;
	UInt2B e_cp;
	UInt2B e_crlc;
	UInt2B e_cparhdr;
	UInt2B e_minalloc;
	UInt2B e_maxalloc;
	UInt2B e_ss;
	UInt2B e_sp;
	UInt2B e_csum;
	UInt2B e_ip;
	UInt2B e_cs;
	UInt2B e_lfarlc;
	UInt2B e_ovno;
	UInt2B e_res[4];
	UInt2B e_oemid;
	UInt2B e_oeminfo;
	UInt2B e_res2[10];
	UInt4B e_lfanew;
};

// See The Old New Thing, Accessing the current modules HINSTANCE from a static library.
import IMAGE_DOS_HEADER __ImageBase;


#if _DEBUG

	void breakOnWindowsError() {
		if(auto const errorID = GetLastError()) {
			//  FORMAT_MESSAGE_ALLOCATE_BUFFER means Win32 allocates the messages memory and expects LocalFree() to be called
			//   to release the buffer.
			//  The error was a system error, so the error message should be searched in system files: FORMAT_MESSAGE_FROM_SYSTEM.
			//  Neither is known what message will come, nor which parameters are expected for formatting. Therefore, do not process
			//   placeholders: FORMAT_MESSAGE_IGNORE_INSERTS.
			//  Do not pass a preferred language (instead, zero). This makes the function search for errors in the following order:
			//      1. Language neutral
			//      2. Thread LANGID, based on the threads locale value
			//      3. User default LANGID, based on the users default locale value
			//      4. System default LANGID, based on the system default locale value
			//      5. US English
			//  Do not insert line breaks (screwing up formatting): FORMAT_MESSAGE_MAX_WIDTH_MASK.
			ASCIICharacter * toZeroTerminatedErrorMessage;
			FormatMessageA(
				0x13FF,
				nullptr,
				errorID,
				0,
				reinterpret_cast<ASCIICharacter *>(&toZeroTerminatedErrorMessage), 0,
				nullptr
			);

			DEBUG_OUTPUT("\nWin32 error: %s\n", toZeroTerminatedErrorMessage);
			OS::Win32::DbgHelp::print(OS::Win32::DbgHelp::currentCallStack(2));
			DEBUG_OUTPUT("\n");

			INVOKE_DEBUGGER;
		}
	}

#endif



// The Page Heap is a default feature of WindowsNT (and of Application Verifier), but it cant be used with Direct3D9 because
//  of buggy AMD video drivers (applications crash at startup already). It sucks, but we need our own implementation.
#if 0
#	include "OS Win32 page heap.inl"
#else
#	if _DEBUG
#		include "OS Win32 debug heap.inl"
#	else
#		include "OS Win32 heap.inl"
#	endif
#endif



namespace OS {


	// Looks up the name of the key at the given scan code, in the language of the current keyboard layout.
	//   requires at least 20 characters
	//   returns a pointer to the new terminating zero (original pointer on failure)
	UTF16Unit * nameOf(IBM::ScanCode const itsScanCode, UTF16Unit * const toResult) {
		return toResult + WIN32_SHOULD_SUCCEED(GetKeyNameTextW(SInt4B(itsScanCode) << 16, toResult, 20));
	}


	namespace Win32 {


		// Reports the last error code to the given supervisor.
		void OUTLINE_CODE reportLastError(Supervisor const * const supervisor) {
			report(supervisor, Error::unspecified, (void const *)UCount(GetLastError()));
		}

		// Prints a description for a Win32 error code according to the current locale.
		//   requires 65536 characters
		//   returns a pointer to the new terminating zero (original pointer on failure)
		UTF16Unit * textFromErrorCode(
			UTF16Unit         text[65535],
			UIntLeast4B const code
		) {
			//  The error was a system error, so the error message should be searched in system files: FORMAT_MESSAGE_FROM_SYSTEM.
			//  Neither is known what message will come, nor which parameters are expected for formatting. Therefore, do not process
			//   placeholders: FORMAT_MESSAGE_IGNORE_INSERTS.
			//  Do not pass a preferred language (instead, zero). This makes the function search for errors in the following order:
			//      1. Language neutral
			//      2. Thread LANGID, based on the threads locale value
			//      3. User default LANGID, based on the users default locale value
			//      4. System default LANGID, based on the system default locale value
			//      5. US English
			//  Do not insert line breaks (screwing up formatting): FORMAT_MESSAGE_MAX_WIDTH_MASK.
			return text + FormatMessageW(
				0x12FF,
				nullptr,
				UInt4B(code),
				0,
				text, 65535,
				nullptr
			);
		}

		// Looks up the name of the given virtual key code, in the language of the current keyboard layout.
		//   requires at least 20 characters
		//   returns a pointer to the new terminating zero (original pointer on failure)
		UTF16Unit * nameOf(VirtualKeyCode itsCode, UTF16Unit * toResult) {
			// Win32 fails on these:
			if(VirtualKeyCode::numpadMultiply == itsCode) { // VK_MULTIPLY
				*toResult++ = UTF16Unit(0xD7); // ה
			} else if(VirtualKeyCode::numpadDivide == itsCode) { // VK_DIVIDE
				*toResult++ = UTF16Unit(0xF7); // 
				itsCode = VirtualKeyCode::numpadMultiply;
			}

			// Win32 does not offer naming of virtual key codes, just of scan codes. Translate.
			auto scanCode = WIN32_SHOULD_SUCCEED(MapVirtualKeyW(UInt4B(itsCode), 0)); // MAPVK_VK_TO_VSC
			// MapVirtualKey(), by default, does not mind the extended key flag. Manually add it for the following keys (see
			//  setnode.com, MapVirtualKey(), GetKeyNameText(), and a story of how to get proper names for the arrow keys):
			//   VK_PRIOR, VK_NEXT
			//   VK_END, VK_HOME
			//   VK_LEFT, VK_UP, VK_RIGHT, VK_DOWN
			//   VK_INSERT, VK_DELETE
			//   VK_NUMLOCK
			if((VirtualKeyCode::pageUp <= itsCode && itsCode <= VirtualKeyCode::downArrow)
			|| VirtualKeyCode::insert == itsCode || VirtualKeyCode::Delete == itsCode
			|| VirtualKeyCode::numLock == itsCode
			) {
				scanCode |= 0x100;
			}

			return OS::nameOf(IBM::ScanCode(scanCode), toResult);
		}



		auto const nullFile = reinterpret_cast<File *>(-1); // INVALID_HANDLE_VALUE

		SECURITY_ATTRIBUTES const inheritable = { sizeof inheritable, nullptr, 1 };


		struct IMAGE_NT_HEADERS {
			UInt4B Signature;
			struct {
				UInt2B Machine;
				UInt2B NumberOfSections;
				UInt4B TimeDateStamp;
				UInt4B PointerToSymbolTable;
				UInt4B NumberOfSymbols;
				UInt2B SizeOfOptionalHeader;
				UInt2B Characteristics;
			}      FileHeader;
			struct {
				UInt2B Magic;
				UInt1B MajorLinkerVersion;
				UInt1B MinorLinkerVersion;
				UInt4B SizeOfCode;
				UInt4B SizeOfInitializedData;
				UInt4B SizeOfUninitializedData;
				UInt4B AddressOfEntryPoint;
				UInt4B BaseOfCode;
#				if COMPILED_FOR_X86_32
					UInt4B BaseOfData;
					UInt4B ImageBase;
#				elif COMPILED_FOR_X86_64
					UInt8B ImageBase;
#				endif
				UInt4B SectionAlignment;
				UInt4B FileAlignment;
				UInt2B MajorOperatingSystemVersion;
				UInt2B MinorOperatingSystemVersion;
				UInt2B MajorImageVersion;
				UInt2B MinorImageVersion;
				UInt2B MajorSubsystemVersion;
				UInt2B MinorSubsystemVersion;
				UInt4B Win32VersionValue;
				UInt4B SizeOfImage;
				UInt4B SizeOfHeaders;
				UInt4B CheckSum;
				UInt2B Subsystem;
				UInt2B DllCharacteristics;
#				if COMPILED_FOR_X86_32
					UInt4B SizeOfStackReserve;
					UInt4B SizeOfStackCommit;
					UInt4B SizeOfHeapReserve;
					UInt4B SizeOfHeapCommit;
#				elif COMPILED_FOR_X86_64
					UInt8B SizeOfStackReserve;
					UInt8B SizeOfStackCommit;
					UInt8B SizeOfHeapReserve;
					UInt8B SizeOfHeapCommit;
#				endif
				UInt4B LoaderFlags;
				UInt4B NumberOfRvaAndSizes;
			}      OptionalHeader;
		};

		// Sets this processs stack reserve and stack commit size to Visual C++ default.
		// This is an ugly hack, but its required because most APIs and drivers create their threads with the default stack size of
		//  the process. If this is very large, hundreds of megabytes are wasted.
		void setDefaultStackOptions() {

			// For historical reasons, all programs begin with a DOS header. Skip to the PE header.
			auto const toPEHeader = reinterpret_cast<IMAGE_NT_HEADERS *>(byteOffset(&__ImageBase, __ImageBase.e_lfanew));

			// Try to change the headers protection to read-execute-write.
			UInt4B originalProtection;
			if(0 != VirtualProtect(toPEHeader, sizeof *toPEHeader, 0x40, &originalProtection)) { // PAGE_EXECUTE_READWRITE

				// Restore Visual C++ default values.
				toPEHeader->OptionalHeader.SizeOfStackReserve = 1024 * 1024;
				toPEHeader->OptionalHeader.SizeOfStackCommit  =    4 * 1024;

				// Restore the pretection.
				UInt4B temp;
				VirtualProtect(toPEHeader, sizeof *toPEHeader, originalProtection, &temp);
			}

		}

		// Returns the amount of virtual memory that Windows has committed for the process.
		USize bytesCommittedForThisProcess() {

			PROCESS_MEMORY_COUNTERS_EX counters;
			if(WIN32_SHOULD_SUCCEED(K32GetProcessMemoryInfo(GetCurrentProcess(), &counters, sizeof counters))) {
				return counters.PrivateUsage;
			}

			return 0;
		}

		// Returns the amount of physical memory installed on the system.
		UInt8B bytesOfPhysicalMemory() {

			MEMORYSTATUSEX status;
			status.dwLength = sizeof status;
			if(WIN32_SHOULD_SUCCEED(GlobalMemoryStatusEx(&status))) {
				return status.ullTotalPhys;
			}

			return ~UInt8B(0); // thats what virtual memory is for: simulating huge RAM
		}

		// Removes as many pages as possible from the working set of this process.
		void minimizeWorkingSet() {
			SetProcessWorkingSetSize(GetCurrentProcess(), ~USize(0), ~USize(0));
		}

		// Removes the given range of pages from the working set of this process.
		void pageOut(void * toMemory, USize itsSize) {
#			if _DEBUG
				// If any of the pages in the specified range are not locked, VirtualUnlock removes such pages from the working set,
				//  sets last error to ERROR_NOT_LOCKED, and returns FALSE.
				if(0 != VirtualUnlock(toMemory, itsSize) || 158 != GetLastError()) {
					INVOKE_DEBUGGER; // The pages must be unlocked before calling this function!
				}
#			else
				VirtualUnlock(toMemory, itsSize);
#			endif
		}





		// A pseudo-variable which always points to the base address of the module the code is compiled to.
		extern Module * const toThisModule = reinterpret_cast<Module *>(&__ImageBase);






		// Returns this processs command line as a zero-terminated string.
		UTF16Unit const * commandLine() {
			auto result = GetCommandLineW(); // cannot fail
			MUST(nullptr != result); // Win32 bug?
			return result;
		}

		// Returns the individual command line arguments of this process as an array of zero-terminated strings.
		//   must be called only once
		Range<UTF16Unit const *> parsedCommandLine() {
#			if _DEBUG
				static bool firstTime = yes;
				MUST(firstTime); // Dont parse the command line twice, save it instead!
				firstTime = no;
#			endif

			UInt4B numberOfArguments = 0;
			auto const result = WIN32_SHOULD_SUCCEED(CommandLineToArgvW(GetCommandLineW(), &numberOfArguments));
			MUST(0 < numberOfArguments); // no Win32 process can have an empty command line

			// This is a memory leak (the result should be freed using LocalFree()), but no programm calls this function more than
			//  once, so we tolerate it here.
			return range(result, numberOfArguments);
		}

		// Writes the path to the current directory of the process.
		//   returns a pointer to the terminating zero
		//   absolute
		//   does not always end with a backslash
		UTF16Unit * currentDirectory(UTF16Unit result[260 + 1]) {
			return result + WIN32_SHOULD_SUCCEED(GetCurrentDirectoryW(261, result));
		}

		// Sets this processs current directory.
		bool currentDirectoryIs(UTF16Unit const * path) {
			return 0 != WIN32_SHOULD_SUCCEED(SetCurrentDirectoryW(path));
		}

		// Exits the calling process normally, performing regular cleanup and detaching DLLs.
		WONT_RETURN exitProcess(UInt4B exitCode) {
			ExitProcess(exitCode); // cannot fail
		}

		// Terminates the calling process immediately without any cleanup or DLL detaching.
		WONT_RETURN terminateProcess(UInt4B exitCode) {
			WIN32_SHOULD_SUCCEED(TerminateProcess(GetCurrentProcess(), exitCode));
			UNREACHABLE;
		}














		// Returns the standard input. Symbolic; do not close!
		File & getStdin() {
			auto result = GetStdHandle(-10); // STD_INPUT_HANDLE
			MUST(nullFile != result && nullptr != result);
			return *result;
		}

		// Returns the standard output. Symbolic; do not close!
		File & getStdout() {
			auto result = GetStdHandle(-11); // STD_OUTPUT_HANDLE
			MUST(nullFile != result && nullptr != result);
			return *result;
		}

		// Returns the standard error output. Symbolic; do not close!
		File & getStderr() {
			auto result = GetStdHandle(-12); // STD_ERROR_HANDLE
			MUST(nullFile != result && nullptr != result);
			return *result;
		}

		void resizeConsole(
			File & console,
			UInt2B newX,
			UInt2B newY
		) {
			MUST(1 <= newX && newX <= 32767);
			MUST(1 <= newY && newY <= 32767);

			PointOnConsole const newSize = { newX, newY };
			WIN32_SHOULD_SUCCEED(SetConsoleScreenBufferSize(&console, newSize));
		}

		File * tryToCreateConsoleRedirection(UTF16Unit const * path) {
			auto result = CreateFileW(
				path,
				0xC0000000, 1, // GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ
				&inheritable,  // in order to use a file as a console redirection, its handle must be inheritable to the new process
				2,             // CREATE_ALWAYS
				256, nullptr   // FILE_ATTRIBUTE_TEMPORARY, reduce flushes to a minimum
			);
			if(nullFile == result) {
				return nullptr;
			}
			return result;
		}

		// Checks whether the given handle is a console. If not, it is redirected (e.g. to a file or pipe).
		local bool isConsole(File * file) {

			if(nullFile == file) {
				return no; // redirected to null
			}

			// For non-console handles, GetConsoleMode() fails with ERROR_INVALID_HANDLE.
			UInt4B dummy;
			if(0 == GetConsoleMode(file, &dummy) && 6 == GetLastError()) {
				return no;
			}

			return yes;
		}


		namespace stdin {


			// Waits for any input to stdin. Most commonly used for press any key to continue in console programs.
			void await() {
				auto const stdin = &getStdin(); // Symbolic; do not close!

				//  screw WaitForSingleObject(): it also returns on focus or mouse events
				//  ReadFile() returns on ENTER (in line mode) or character input
				//  discard any excess input before it pollutes subsequent calls:
				//    in line mode, the user might produce arbitrary input before hitting ENTER
				//    ENTER produces CR+LF
				Byte   buffer = 0;
				UInt4B numberOfBytesRead;
				do {
					WIN32_SHOULD_SUCCEED(ReadFile(stdin, &buffer, 1, &numberOfBytesRead, nullptr));
				} until('\n' == buffer);
			}

			// Reads from stdin until a line break is hit.
			//   result is always zero-terminated (zero must be included in size)
			//   returns pointer to terminating zero
			//   excess input is discarded
			//   non-ASCII characters come out undefined
			//   on failure, returns the original pointer
			ASCIICharacter * readLine(ASCIICharacter * toResult, UCount remainingCharacters) {
				auto const stdin = &getStdin(); // Do not close (symbolic)! Do not fetch (my be redirected between calls)!
				UCount     totalCharactersRead = 0;

				// Easy:
				//   ReadFile() translates all carriage returns to CR+LF.
				//   There will never be more than one CR+LF read by ReadFile(), independent of console mode. Even if a user types a
				//    raw CR (via ALT+013), ReadFile() returns immediately with CR+LF.
				// Hard:
				//   ReadConsole() cannot be used because it doesnt support redirected input handles.
				//   Short reads might occur: ReadFile() returning fewer bytes than requested. This is not an error (it allows
				//    transparent block-based compression) and the application should just keep reading. See The Old New Thing,
				//    Changing the conditions under which ReadFile produces fewer bytes than requested.
				//   Memory limit: Console reads fail for more than 26608/31004/31346 B (on WindowsXP/Server 2003/7). So, never
				//    request more than safe 16 KB at once (see gdb: Windows semihosting (remote-fileio) fix for console reads).
				//   The input might exceed the buffer. If so, all remaining input needs to be read (until CR+LF is reached), or
				//    subsequent calls will be polluted with excess data.
				cycle {
					UInt4B     charactersRead; // ReadFile sets this value to zero before doing any work or error checking.
					auto const charactersToRead = UInt4B(minimumOf(remainingCharacters, UCount(16384)));
					auto       result = ReadFile(stdin, toResult + totalCharactersRead, charactersToRead, &charactersRead, nullptr);
					totalCharactersRead += charactersRead;
					remainingCharacters -= charactersRead;

					if(0 == result) { // Read error?
						WIN32_SHOULD_HAVE_SUCCEEDED;
						*toResult = 0;
						return toResult;
					}

					if(totalCharactersRead >= 2 && '\r' == toResult[totalCharactersRead - 2] && '\n' == toResult[totalCharactersRead - 1]) { // End of input?
						auto toEnd = toResult + totalCharactersRead - 2;
						*toEnd = 0;
						return toEnd;
					}

					if(0 == remainingCharacters) { // Input exceeds buffer?
						auto toEnd = toResult + totalCharactersRead - 1;

						// Read until the next CR+LF is hit. Consider the special case where CR is already copied to the buffer, but LF is not.
						//  If this fails, dont care  we dont want it anyway.
						until('\r' == *toEnd) {
							WIN32_SHOULD_SUCCEED(ReadFile(stdin, toEnd, 1, &charactersRead, nullptr));
						}

						// Overwrite with the remaining LF:
						WIN32_SHOULD_SUCCEED(ReadFile(stdin, toEnd, 1, &charactersRead, nullptr));
						MUST('\n' == *toEnd);

						// Replace with the terminating zero:
						*toEnd = 0;
						return toEnd;
					}

				}
			}

			// Reads from stdin until a line break is hit.
			//   result is always zero-terminated (zero must be included in size)
			//   returns pointer to terminating zero
			//   excess input is discarded
			//   on failure, returns the original pointer
			UTF16Unit * readLine(UTF16Unit * toResult, UCount remainingCharacters) {
				auto const stdin = &getStdin(); // Do not close (symbolic)! Do not fetch (my be redirected between calls)!
				if(isConsole(stdin)) { // ReadConsoleW(), the only Unicode console API, does not support redirection

					// Easy:
					//   ReadConsoleW() translates all carriage returns to CR+LF.
					//   There will never be more than one CR+LF read by ReadConsoleW(), independent of console mode. Even if a user
					//    types a raw CR (via ALT+013), ReadConsoleW() returns immediately with CR+LF.
					// Hard:
					//   Short reads might occur: ReadConsoleW() returning fewer bytes than requested. This is not an error (it allows
					//    transparent block-based compression) and the application should just keep reading. See The Old New Thing,
					//    Changing the conditions under which ReadFile produces fewer bytes than requested.
					//   Memory limit: Console reads fail for more than 26608/31004/31346 B (on WindowsXP/Server 2003/7). So, never
					//    request more than safe 16 KB at once (see gdb: Windows semihosting (remote-fileio) fix for console reads).
					//   The input might exceed the buffer. If so, all remaining input needs to be read (until CR+LF is reached), or
					//    subsequent calls will be polluted with excess data.
					USize totalCharactersRead = 0;
					cycle {
						UInt4B charactersRead;
						auto result = ReadConsoleW(stdin, toResult + totalCharactersRead, UInt4B(minimumOf(remainingCharacters, UCount(8192))), &charactersRead, nullptr);
						totalCharactersRead += charactersRead;
						remainingCharacters -= charactersRead;

						if(0 == result) { // Read error?
							WIN32_SHOULD_HAVE_SUCCEEDED;
							*toResult = 0;
							return toResult;
						}

						if(totalCharactersRead >= 2 && '\r' == toResult[totalCharactersRead - 2] && '\n' == toResult[totalCharactersRead - 1]) { // End of input?
							auto toEnd = toResult + totalCharactersRead - 2;
							*toEnd = 0;
							return toEnd;
						}

						if(0 == remainingCharacters) { // Input exceeds buffer?
							auto toEnd = toResult + totalCharactersRead - 1;

							// Read until the next CR+LF is hit. Consider the special case where CR is already copied to the buffer, but LF is
							//  not. If this fails, dont care  we dont want it anyway.
							until('\r' == *toEnd) {
								WIN32_SHOULD_SUCCEED(ReadFile(stdin, toEnd, 1, &charactersRead, nullptr));
							}

							// Overwrite with the remaining LF:
							WIN32_SHOULD_SUCCEED(ReadFile(stdin, toEnd, 1, &charactersRead, nullptr));
							MUST('\n' == *toEnd);

							// Replace with the terminating zero:
							*toEnd = 0;
							return toEnd;
						}

					}

				} else {

					INVOKE_DEBUGGER; // TODO: Implement!
					return nullptr;

				}
			}


		} // stdin













		// Loads a system standard cursor of the given MAKEINTRESOURCE ID.
		template <UCount id> local Icon & systemCursor() {
			return *WIN32_SHOULD_SUCCEED(LoadCursorW(nullptr, id));
		}

		Icon & systemCursor_arrow() {
			return systemCursor<32512>(); // IDC_ARROW
		}
		Icon & systemCursor_cross() {
			return systemCursor<32515>(); // IDC_CROSS
		}
		Icon & systemCursor_sizeLeftRight() {
			return systemCursor<32644>(); // IDC_SIZEWE
		}
		Icon & systemCursor_sizeUpDown() {
			return systemCursor<32645>(); // IDC_SIZENS
		}
		Icon & systemCursor_sizeAll() {
			return systemCursor<32646>(); // IDC_SIZEALL
		}
		Icon & systemCursor_busy() {
			return systemCursor<32514>(); // IDC_WAIT
		}
		Icon & systemCursor_busyInBackground() {
			return systemCursor<32650>(); // IDC_APPSTARTING
		}

		void cursorPositionIs(PointOnScreen const & position) {
			WIN32_SHOULD_SUCCEED(SetCursorPos(position.x, position.y));
		}

		void curserIconIs(Icon & icon) {
			SetCursor(&icon);
		}

		void curserIconIsBlank() {
			SetCursor(nullptr);
		}






		// Copies the text in the clipboard to the given zero-terminated UTF-16 string.
		//   returns no on failure or if the destination is too small
		bool copyFromClipboard(UTF16Unit * toResult, UTF16Unit * toEndOfResult) {
			bool result = no;

			// Open the clipboard. Since no delayed rendering is used, no owner is given.
			if(0 == WIN32_SHOULD_SUCCEED(OpenClipboard(nullptr))) {
				return result;
			}

			// Get the clipboard data. This is NULL if the clipboard is empty (or of different format).
			if(auto storage = WIN32_SHOULD_SUCCEED(GetClipboardData(13))) { // CF_UNICODETEXT

				// Map the global memory to virtual memory:
				if(auto toData = reinterpret_cast<UTF16Unit const *>(WIN32_SHOULD_SUCCEED(GlobalLock(storage)))) {

					auto const length = Text::lengthOf(toData);
					if(length + 1 < UCount(toEndOfResult - toResult)) {
						Text::copyZeroTerminated(toResult, toData);
						result = yes;
					}

					GlobalUnlock(storage);
				}

			}

			WIN32_SHOULD_SUCCEED(CloseClipboard());

			return result;
		}

		// Copies the given zero-terminated ASCII string to the clipboard.
		//   returns no on failure
		//   line breaks must be carriage return + line feed to work properly
		bool copyToClipboard(ASCIICharacter const * text) {
			bool result = no;

			// Clipboard content of CF_TEXT format must be a zero-terminated ANSI string.
			auto const size = Text::lengthOf(text) + 1;

			// Open the clipboard and clean up previous content. Since no delayed rendering is used, no owner is given.
			if(0 == WIN32_SHOULD_SUCCEED(OpenClipboard(nullptr))) {
				return result;
			}
			WIN32_SHOULD_SUCCEED(EmptyClipboard());

			// Allocate a global memory block:
			if(auto storage = WIN32_SHOULD_SUCCEED(GlobalAlloc(0x42, size))) { // GMEM_MOVEABLE | GMEM_ZEROINIT

				// Map the global memory to virtual memory:
				if(auto toData = reinterpret_cast<ASCIICharacter *>(WIN32_SHOULD_SUCCEED(GlobalLock(storage)))) {

					// Fill it:
					Text::copyZeroTerminated(toData, text);
					WIN32_SHOULD_SUCCEED(GlobalUnlock(storage));

					// Submit the global memory to the clipboard. Only on success, Win32 becomes responsible for cleanup.
					if(nullptr != WIN32_SHOULD_SUCCEED(SetClipboardData(1, storage))) { // CF_TEXT
						result = yes;
					} else {
						GlobalFree(storage);
					}
				}

			}

			WIN32_SHOULD_SUCCEED(CloseClipboard());

			return result;
		}

		// Copies the given zero-terminated UTF-16 string to the clipboard.
		//   returns no on failure
		//   line breaks must be carriage return + line feed to work properly
		bool copyToClipboard(UTF16Unit const * text) {
			bool result = no;

			// Clipboard content of CF_UNICODETEXT format must be a zero-terminated ANSI string.
			auto const size = sizeof(UTF16Unit) * (Text::lengthOf(text) + 1);

			// Open the clipboard and clean up previous content. Since no delayed rendering is used, no owner is given.
			if(0 == WIN32_SHOULD_SUCCEED(OpenClipboard(nullptr))) {
				return result;
			}
			WIN32_SHOULD_SUCCEED(EmptyClipboard());

			// Allocate a global memory block:
			if(auto storage = WIN32_SHOULD_SUCCEED(GlobalAlloc(0x42, size))) { // GMEM_MOVEABLE | GMEM_ZEROINIT

				// Map the global memory to virtual memory:
				if(auto toData = reinterpret_cast<UTF16Unit *>(WIN32_SHOULD_SUCCEED(GlobalLock(storage)))) {

					// Fill it:
					Text::copyZeroTerminated(toData, text);
					WIN32_SHOULD_SUCCEED(GlobalUnlock(storage));

					// Submit the global memory to the clipboard. Only on success, Win32 becomes responsible for cleanup.
					if(nullptr != WIN32_SHOULD_SUCCEED(SetClipboardData(13, storage))) { // CF_UNICODETEXT
						result = yes;
					} else {
						GlobalFree(storage);
					}
				}

			}

			WIN32_SHOULD_SUCCEED(CloseClipboard());

			return result;
		}

		// Copies the given bitmap to the clipboard.
		//   returns no on failure
		bool copyToClipboard_bottomUp(
			UIntLeast2B             widthInPixels,
			UIntLeast2B             heightInPixels,
			GFX::SBGRC_8888 const * pixels
		) {
			bool result = no;

			// Clipboard content of CF_UNICODETEXT consists of a v3 bitmap header and the pixels.
			auto const imageSize = widthInPixels * heightInPixels * sizeof(*pixels);
			auto const size      = sizeof(Formats::BMP::Header_v3_NT) + imageSize;

			// Open the clipboard and clean up previous content. Since no delayed rendering is used, no owner is given.
			if(0 == WIN32_SHOULD_SUCCEED(OpenClipboard(nullptr))) {
				return result;
			}
			WIN32_SHOULD_SUCCEED(EmptyClipboard());

			// Allocate a global memory block:
			if(auto storage = WIN32_SHOULD_SUCCEED(GlobalAlloc(0x42, size))) { // GMEM_MOVEABLE | GMEM_ZEROINIT

				// Map the global memory to virtual memory:
				if(auto toBMP = reinterpret_cast<Formats::BMP::Header_v3_NT *>(WIN32_SHOULD_SUCCEED(GlobalLock(storage)))) {

					// Fill it:
					toBMP->sizeInBytes      = sizeof *toBMP;
					toBMP->widthInPixels    = widthInPixels;
					toBMP->heightInPixels   = heightInPixels;
					toBMP->one              = 1;
					toBMP->bitsPerPixel     = 32;
					toBMP->compression      = 0;
					toBMP->imageSizeInBytes = UInt4B(imageSize);
					toBMP->pixelsPerMeterX  = 0;
					toBMP->pixelsPerMeterY  = 0;
					toBMP->colorCount       = 0;
					toBMP->colorsImportant  = 0;
					movs((GFX::SBGRC_8888 *)byteOffset(toBMP, sizeof *toBMP), (GFX::SBGRC_8888 const *)pixels, widthInPixels * heightInPixels);

					WIN32_SHOULD_SUCCEED(GlobalUnlock(storage));

					// Submit the global memory to the clipboard. Only on success, Win32 becomes responsible for cleanup.
					if(nullptr != WIN32_SHOULD_SUCCEED(SetClipboardData(8, storage))) { // CF_DIB
						result = yes;
					} else {
						GlobalFree(storage);
					}
				}

			}

			WIN32_SHOULD_SUCCEED(CloseClipboard());

			return result;
		}






		void enableStandbyForSystemAndScreen() {
			SetThreadExecutionState(0);
		}
		void disableStandbyForSystemAndScreen() {
			SetThreadExecutionState(0x80000003); // ES_CONTINUOUS | ES_SYSTEM_REQUIRED | ES_DISPLAY_REQUIRED
		}










		Ticks performanceCounterFrequencyInHz() {
			UInt8B result;
			// Under what circumstances does QueryPerformanceFrequency return FALSE?  This wont occur on any system that runs
			//  WindowsXP or later. (MSDN, Acquiring high-resolution time stamps)
			WIN32_SHOULD_SUCCEED(QueryPerformanceFrequency(&result));
			return result;
		}

		// Returns the current value of the high-performance counter.
		Ticks performanceCounter() {
			UInt8B result;
			// Under what circumstances does QueryPerformanceCounter return zero?  This wont occur on any system that runs
			//  WindowsXP or later. (MSDN, Acquiring high-resolution time stamps)
			WIN32_SHOULD_SUCCEED(QueryPerformanceCounter(&result));
			return result;
		}

		// Returns the CPUs internal time stamp counter.
		UInt8B cpuCounter() {
#			if COMPILED_BY_VISUAL_CPP
				return __rdtsc();
#			elif COMPILED_BY_CLANG
				return __builtin_readcyclecounter();
#			else
#				error TODO
#			endif
		}

		// Prints the current date & time (Coordinated Universal Time) as YYYY-MM-DD hh-mm-ss. Useful for logs and unique file names.
		//   returns a pointer to the strings end
		UTF16Unit * writeCurrentDateAndTime(UTF16Unit result[19]) {
			auto toEnd = result;

			SYSTEMTIME time;
			GetSystemTime(&time);

			*toEnd++ = '2';
			*toEnd++ = '0';
			*toEnd++ = Text::toDecimalDigit(time.wYear / 10 % 10);
			*toEnd++ = Text::toDecimalDigit(time.wYear % 10);
			*toEnd++ = '-';
			*toEnd++ = Text::toDecimalDigit(time.wMonth / 10 % 10);
			*toEnd++ = Text::toDecimalDigit(time.wMonth % 10);
			*toEnd++ = '-';
			*toEnd++ = Text::toDecimalDigit(time.wDay / 10 % 10);
			*toEnd++ = Text::toDecimalDigit(time.wDay % 10);
			*toEnd++ = ' ';
			*toEnd++ = Text::toDecimalDigit(time.wHour / 10 % 10);
			*toEnd++ = Text::toDecimalDigit(time.wHour % 10);
			*toEnd++ = '-';
			*toEnd++ = Text::toDecimalDigit(time.wMinute / 10 % 10);
			*toEnd++ = Text::toDecimalDigit(time.wMinute % 10);
			*toEnd++ = '-';
			*toEnd++ = Text::toDecimalDigit(time.wSecond / 10 % 10);
			*toEnd++ = Text::toDecimalDigit(time.wSecond % 10);

			return toEnd;
		}


		namespace Settings {

			// Returns the factor for the mouse speed (which can be customized through the mouse Control Panel application).
			//   in the range of 1 (slowest) to 20 (fastest)
			//   on failure, returns the default of 10
			UIntLeast4B tryToGetMouseSpeedMultiplier() {
				UInt4B result;
				if(0 == WIN32_SHOULD_SUCCEED(SystemParametersInfoW(112, 0, &result, 0))) { // SPI_GETMOUSESPEED
					return 10;
				}
				return result;
			}

		} // Settings





		// Opens an existing pipe.
		//   the name must be zero-terminated and shorter than 249 characters
		//   returns no on failure
		//   must be close()d otherwise
		//   must be located on the local system
		//   opened for reading & writing
		File * tryToOpenLocalDuplexPipe(UTF16Unit const * const name) {
			MUST(Text::lengthOf(name) < 250); // the entire path must not exceed 260 characters

			// Assemble the pipes path:
			UTF16Unit path[260 + 1];
			Text::copyZeroTerminated(Text::copyUntilZero(path, "\\\\.\\pipe\\"), name);

			// Open the pipe:
			auto result = CreateFileW(
				path,
				0xC0000000, 0, nullptr, // GENERIC_READ | GENERIC_WRITE
				3, // OPEN_EXISTING
				0, nullptr
			);
			if(nullFile == result) {
				WIN32_SHOULD_HAVE_SUCCEEDED;
			 return nullptr;
			}
			return result;
		}

		bool tryToCreatePipeForChildProcess(
			File * & out,
			File * & in
		) {
			return 0 != WIN32_SHOULD_SUCCEED(CreatePipe(&out, &in, &inheritable, 0));
		}


		// Captures the current call stack (excluding this call) as a zero-terminated string.
		//   writes an empty string on failure
		//   layout:
		//     our program.exe                  + 0x12345678
		//     <unknown module>
		//     Kernel32.dll                     + 0x12345678
		//     MaximalLengthIs32CharactersPerF + 0x12345678
		//     
		//   up to 16 entries
		void captureCallStack(UTF16Unit result[16 * 48]) {
			auto toEndOfText = result;

			// Capture the raw call stack:
			void * stackPointers[16];
			auto       toCall = stackPointers;
			auto const toEnd  = toCall + RtlCaptureStackBackTrace(1, CAPACITY_OF(stackPointers), stackPointers, nullptr);
			while(toCall < toEnd) {

				// The addresses are useless with Address Space Layout Randomization  instead, get an offset into the module:
				//   GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS allows to pass a code address instead of a string
				//   GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT prevents taking a new reference to the module
				Module * toModule;
				if(WIN32_SHOULD_SUCCEED(GetModuleHandleExW(6, (UTF16Unit const *)*toCall, &toModule))) {

					// Get the modules name. Consider:
					//   WindowsXP does not zero-terminate long paths, so allocate an additional character and zero it manually
					UTF16Unit path[260];
#					if TARGET_OS_VERSION <= TARGET_OS_VERSION_WINDOWS_XP
						auto const toEndOfName = path + WIN32_SHOULD_SUCCEED(GetModuleFileNameW(toModule, path, CAPACITY_OF(path) - 1));
						*toEndOfName = 0;
#					else
						auto const toEndOfName = path + WIN32_SHOULD_SUCCEED(GetModuleFileNameW(toModule, path, CAPACITY_OF(path)));
#					endif

					// From the paths end, seek backwards until encountering either
					//   the paths beginning
					//   or a forward or backward slash
					auto toName = toEndOfName;
					while(toName > toBeginningOf(path) && *toName != UTF16('\\') && *toName != UTF16('/')) {
						--toName;
					}
					if(*toName == UTF16('\\') || *toName == UTF16('/')) {
						++toName;
					}
					auto const nameLength = toEndOfName - toName;

					// Copy the name:
					if(nameLength < 32) {
						toEndOfText = Text::copyUntilZero(toEndOfText, toName);
						toEndOfText = fill(toEndOfText, UTF16(' '), 32 - nameLength);
					} else {
						toEndOfText = movs(toEndOfText, toName, 31);
						*toEndOfText++ = UTF16Unit(0x85); // 
					}

					// Compute the calls offset in the module:
					//   always 4-B because even 64-bit Portable Executables are limited to 2 bytes
					auto const offset = UInt4B(byteDistance((void *)toModule, *toCall));

					// Write the offset:
					*toEndOfText++ = '+';
					toEndOfText = Text::toHexadecimalZeroPaddedString(toEndOfText, offset);
					*toEndOfText++ = '\n';

				} else {
					toEndOfText = Text::copyZeroTerminated(toEndOfText, "<unknown module>\n");
				}

				++toCall;
			}

			*toEndOfText++ = 0; // zero-terminate the string
		}


		// Tries to detect the codepage of the given data.
		//   returns CodePage::unknown on failure
		CodePage detectCodePage(
			Byte const   data[],
			UCount const length
		) {
			// Make an educated guess:
			//   UTF-8 if any of the following bit patterns occur:
			//     110xxxxx 10xxxxxx
			//     1110xxxx 10xxxxxx 10xxxxxx
			//     11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
			//   Cyrillic if 75 % or more of the characters are in the range of C0FF.
			UCount     aboveC0     = 0;
			auto       hasUTF8     = no;
			auto const toEndOfData = data + length;
			do {
				if(0xC0 <= *data) {
					++aboveC0;
				}
				if(0xC0 == (data[0] & 0xE0) && 0x80 == (data[1] & 0xC0)) {
					hasUTF8 = yes;
				} else if(0xE0 == (data[0] & 0xF0) && 0x80 == (data[1] & 0xC0) && 0x80 == (data[2] & 0xC0)) {
					hasUTF8 = yes;
				} else if(0xF0 == (data[0] & 0xF8) && 0x80 == (data[1] & 0xC0) && 0x80 == (data[2] & 0xC0) && 0x80 == (data[3] & 0xC0)) {
					hasUTF8 = yes;
				}
			} while(++data < toEndOfData);
			auto const hasCyrillic = aboveC0 >= length * 3 / 4;
			if(hasUTF8) {
				return CodePage::utf8;
			} else if(hasCyrillic) {
				return CodePage::cyrillic_1251;
			} else {
				return CodePage::unknown; // use the system default code page
			}
		}


		// Converts the given characters from the given code page to UTF-16.
		//   the UTF-16 buffer should be four times as long as the ANSI buffer (accounts for the worst case)
		//   returns the end of the result
		UTF16Unit * utf16FromCodePage(
			CodePage const               codepage,
			UTF16Unit * const            toResult,
			ASCIICharacter const * const toANSI,
			UCount const                 lengthOfANSI
		) {
			MUST(nullptr != toResult);
			MUST(nullptr != toANSI);
			MUST(lengthOfANSI <= 0x7FFF'FFFF / 4);

			auto const length = int(lengthOfANSI);
			return toResult + MultiByteToWideChar(
				UInt4B(codepage), 0,
				toANSI, length,
				toResult, 4 * length
			);
		}
		UTF16Unit * utf16FromCodePage(
			CodePage const               codepage,
			UTF16Unit * const            toResult,
			ASCIICharacter const * const toANSI,
			ASCIICharacter const * const toEndOfANSI
		) {
			MUST(toANSI <= toEndOfANSI);

			return utf16FromCodePage(codepage, toResult, toANSI, UCount(toEndOfANSI - toANSI));
		}


	} // Win32


} // OS