#include "framework.hpp"
#include "Parallel.hpp"



#if COMPILED_BY_VISUAL_CPP

	// Visual C++ intrinsics. On Clang and GCC, the __builtin_ counterparts are used, so no declaration is required.
	import {

		void _mm_pause();

		long _InterlockedCompareExchange(long volatile * Destination, long Exchange, long Comparand);
#		pragma intrinsic(_InterlockedCompareExchange)
		long _InterlockedIncrement(long volatile *);
#		pragma intrinsic(_InterlockedIncrement)
		long _InterlockedDecrement(long volatile *);
#		pragma intrinsic(_InterlockedDecrement)
		long _InterlockedExchange(long volatile *, long);
#		pragma intrinsic(_InterlockedExchange)

#		if COMPILED_FOR_X86_64
			long long _InterlockedIncrement64(long long volatile *);
#			pragma intrinsic(_InterlockedIncrement64)
			long long _InterlockedDecrement64(long long volatile *);
#			pragma intrinsic(_InterlockedDecrement64)
#		endif
	}

#endif



// If the given value equals zero, the given replacement is written and yes is returned. Otherwise, no is returned.
//  The operation is atomic.
local bool atomicExchangeIfZero(
	UInt4B volatile & value,
	UInt4B const      replacementIfZero
) {
#	if COMPILED_BY_VISUAL_CPP
		return 0 == _InterlockedCompareExchange(reinterpret_cast<long volatile *>(&value), long(replacementIfZero), 0);
#	elif COMPILED_BY_CLANG
		return __sync_bool_compare_and_swap(&value, 0, replacementIfZero);
#	else
#		error TODO
#	endif
}

// Increments the given number atomically and returns the new value.
UInt4B atomicIncrement(UInt4B volatile & value) {
#	if COMPILED_BY_VISUAL_CPP
		return _InterlockedIncrement(reinterpret_cast<long volatile *>(&value));
#	elif COMPILED_BY_CLANG
		return __sync_add_and_fetch(&value, 1);
#	else
#		error TODO
#	endif
}
#if COMPILED_FOR_X86_64
	UInt8B atomicIncrement(UInt8B volatile & value) {
#		if COMPILED_BY_VISUAL_CPP
			return _InterlockedIncrement64(reinterpret_cast<long long volatile *>(&value));
#		elif COMPILED_BY_CLANG
			return __sync_add_and_fetch(&value, 1);
#		else
#			error TODO
#		endif
	}
#endif

// Decrements the given number atomically and returns the new value.
UInt4B atomicDecrement(UInt4B volatile & value) {
#	if COMPILED_BY_VISUAL_CPP
		return _InterlockedDecrement(reinterpret_cast<long volatile *>(&value));
#	elif COMPILED_BY_CLANG
		return __sync_sub_and_fetch(&value, 1);
#	else
#		error TODO
#	endif
}
#if COMPILED_FOR_X86_64
	UInt8B atomicDecrement(UInt8B volatile & value) {
#		if COMPILED_BY_VISUAL_CPP
			return _InterlockedDecrement64(reinterpret_cast<long long volatile *>(&value));
#		elif COMPILED_BY_CLANG
			return __sync_sub_and_fetch(&value, 1);
#		else
#			error TODO
#		endif
	}
#endif

// Replaces the given number atomically and returns the previous value.
UInt4B atomicExchange(UInt4B volatile & value, UInt4B const newValue) {
#	if COMPILED_BY_VISUAL_CPP
		return _InterlockedExchange(reinterpret_cast<long volatile *>(&value), newValue);
#	elif COMPILED_BY_CLANG
		return __sync_lock_test_and_set(&value, 1);
#	else
#		error TODO
#	endif
}

void lock(SpinLock & spinLock) {
	until(atomicExchangeIfZero(spinLock.lock, 1)) {
		// Use x86 PAUSE for reduced power consumption and optimized pipelining while spinlocking:
#		if COMPILED_BY_VISUAL_CPP
			_mm_pause(); 
#		elif COMPILED_BY_CLANG
			__asm__ volatile ("pause");
#		elif COMPILED_BY_GCC
			__asm__ __volatile__ ("rep; nop" : : );
#		else
#			error TODO
#		endif
	}
	MUST(0 != spinLock.lock);
}

void unlock(SpinLock & spinLock) {
	MUST(0 != spinLock.lock);
	spinLock.lock = 0;
}