// Interoperability with the PlayStation, Sonys 1995 video game console.



#include "framework.hpp"
#include "Sony PSX.hpp"



namespace Sony {


	namespace PSX {


		// X coordinate of the texture page, in frame buffer pixels (i.e. 2-B words).
		//   always aligned to 64 pixels
		UIntLeast2B DrawAttribute_Texture::xInFrameBufferPixels() const {
			return 64 * (littleEndian(data) & 15);
		}

		// Y coordinate of the texture page, in pixels.
		//   always aligned to 256 pixels
		UIntLeast2B DrawAttribute_Texture::yInPixels() const {
			return 16 * (littleEndian(data) & 16);
		}

		// The textures bit depth.
		//   4: 16 indexed colors
		//   8: 256 indexed colors
		//   16: High Color with transparency
		UIntLeast1B DrawAttribute_Texture::bitsPerPixel() const {
			switch(littleEndian(data) & 192) {
				case   0: return 4;
				case  64: return 8;
				case 128: return 16;
				case 192: INVOKE_DEBUGGER; UNREACHABLE; // TODO: Reserved value; how to handle?
			}
		}


		// X coordinate, in frame buffer pixels (i.e. 2-B words).
		//   always aligned to 16 pixels (in accordance with PSX GPU requirements)
		UIntLeast2B DrawAttribute_Palette::xInFrameBufferPixels() const {
			return 16 * UIntLeast2B(littleEndian(data) & 63);
		}

		// Y coordinate, in pixels.
		UIntLeast2B DrawAttribute_Palette::yInPixels() const {
			return 511 & (UIntLeast2B(littleEndian(data)) >> 6);
		}


	} // PSX


} // Sony