namespace Text {


	// Returns the number of characters in the given zero-terminated string.
	UCount lengthOf(ASCIICharacter const * toZeroTerminatedASCIIString);
	UCount lengthOf(UTF16Unit const * toZeroTerminatedUTF16String);

	// Returns a pointer to the terminating zero in the given string.
	ASCIICharacter const * seekEndOf(ASCIICharacter const * toZeroTerminatedASCIIString);
	ASCIICharacter * seekEndOf(ASCIICharacter * toZeroTerminatedASCIIString);
	UTF16Unit const * seekEndOf(UTF16Unit const * toZeroTerminatedUTF16String);
	UTF16Unit * seekEndOf(UTF16Unit * toZeroTerminatedUTF16String);

	// Creates a range from the given zero-terminated string.
	Range<ASCIICharacter const> range(ASCIICharacter const * toZeroTerminatedASCIIString);
	Range<UTF16Unit const> range(UTF16Unit const * toZeroTerminatedUTF16String);

	// Creates a range from the given zero-terminated string.
	Range<ASCIICharacter> range(ASCIICharacter * toZeroTerminatedASCIIString);

	// (complexity: linear in length of string)
	ASCIICharacter * copyZeroTerminated(ASCIICharacter *, ASCIICharacter const *);
	ASCIICharacter * copyZeroTerminated(ASCIICharacter *, UTF16Unit const *); // replaces non-ASCII characters with question marks
	UTF16Unit * copyZeroTerminated(UTF16Unit *, ASCIICharacter const *);
	UTF16Unit * copyZeroTerminated(UTF16Unit *, UTF8Unit const *);
	UTF16Unit * copyZeroTerminated(UTF16Unit *, UTF16Unit const *);
	UTF8Unit * copyZeroTerminated(UTF8Unit *, ASCIICharacter const *);
	UTF8Unit * copyZeroTerminated(UTF8Unit *, UTF8Unit const *);

	// (complexity: linear in length of string)
	ASCIICharacter * copy(
		ASCIICharacter *                    toDestination,
		Range<ASCIICharacter const> const & source
	);
	ASCIICharacter * copy(
		ASCIICharacter *       toDestination,
		ASCIICharacter const * toSource,
		ASCIICharacter const * toEndOfSource
	);
	UTF16Unit * copy(
		UTF16Unit *       toDestination,
		UTF16Unit const * toSource,
		UCount
	);
	ASCIICharacter * copy(
		ASCIICharacter *       toDestination,
		ASCIICharacter const * toSource,
		UCount
	);
	ASCIICharacter * copyAndAppendZero(
		ASCIICharacter *                    toDestination,
		Range<ASCIICharacter const> const & source
	);
	ASCIICharacter * copyAndAppendZero(
		ASCIICharacter *       toDestination,
		ASCIICharacter const * toSource,
		ASCIICharacter const * toEndOfSource
	);
	UTF16Unit * copyAndAppendZero(
		UTF16Unit *                    toDestination,
		Range<UTF16Unit const> const & source
	);
	UTF16Unit * copyAndAppendZero(
		UTF16Unit *       toDestination,
		UTF16Unit const * toSource,
		UTF16Unit const * toSourceEnd
	);
	ASCIICharacter * copyAndAppendZero(
		ASCIICharacter *       toDestination,
		ASCIICharacter const * toSource,
		UCount                 sourcesLength
	);
	UTF16Unit * copyAndAppendZero(
		UTF16Unit *            toDestination,
		ASCIICharacter const * toSource,
		UCount                 sourcesLength
	);
	ASCIICharacter * copyUntilZero(
		ASCIICharacter *       toDestination,
		ASCIICharacter const * toSource
	);
	UTF16Unit * copyUntilZero(
		UTF16Unit *            toDestination,
		ASCIICharacter const * toSource
	);
	UTF16Unit * copyUntilZero(
		UTF16Unit *       toDestination,
		UTF16Unit const * toSource
	);

	bool isLowerCaseAlphabetical(ASCIICharacter);

	bool isUpperCaseAlphabetical(ASCIICharacter);

	bool isAlphabetical(ASCIICharacter);

	bool isDecimalDigit(ASCIICharacter);
	bool isDecimalDigit(UTF16Unit);
	bool isLowerCaseHexadecimalDigit(ASCIICharacter);
	bool isUpperCaseHexadecimalDigit(ASCIICharacter);
	bool isOctalDigit(ASCIICharacter);

	// Checks whether the given ASCII character is a control character (zero terminator, bell, line feed, ).
	bool isControlCharacter(ASCIICharacter);

	// Checks whether the given UTF-16 code unit is a ASCII control character (zero terminator, bell, line feed, ) but NOT a
	//  Unicode control character (e.g. right-to-left control characters).
	bool isASCIIControlCharacter(UTF16Unit);

	ASCIICharacter toDecimalDigit(UIntLeast1B);

	UIntLeast1B decimalDigitToNumber(ASCIICharacter);
	UIntLeast1B decimalDigitToNumber(UTF16Unit);

	// Returns zero for the ASCII character 0, one for 1, etc.
	//   returns a number larger than 9 if the character was no decimal digit
	UIntLeast1B numberFromDecimalDigit_unchecked(ASCIICharacter);
	UIntLeast2B numberFromDecimalDigit_unchecked(UTF16Unit);

	// Returns zero for the ASCII character 0, one for 1, etc.
	//   returns a number larger than 7 if the character was no octal digit
	UIntLeast1B numberFromOctalDigit_unchecked(ASCIICharacter);
	UIntLeast2B numberFromOctalDigit_unchecked(UTF16Unit);

	UIntLeast1B numberFromUpperCaseHexadecimalDigit(ASCIICharacter);

	UIntLeast1B numberFromOctalDigit(ASCIICharacter);

	ASCIICharacter toHexadecimalDigit(UIntLeast1B);

	bool isWhitespace(ASCIICharacter);
	bool isWhitespace(UTF16Unit);

	bool isLineBreak(ASCIICharacter);
	bool isLineBreak(UTF16Unit);

	bool isWhitespaceButNotLineBreak(ASCIICharacter);

	// Returns the upper case version of the given character.
	ASCIICharacter upperCase(ASCIICharacter);
	UTF16Unit upperCase(UTF16Unit);

	// Returns the lower case version of the given character.
	ASCIICharacter lowerCase(ASCIICharacter);
	UTF16Unit lowerCase(UTF16Unit);

	// Lowers the case of the given string.
	ASCIICharacter * lowerCase(
		ASCIICharacter *       toDestination,
		ASCIICharacter const * toSource,
		ASCIICharacter const * toSourcesEnd
	);

	// Determines whether the given strings equal.
	bool equal(
		Range<ASCIICharacter const> const &,
		Range<ASCIICharacter const> const &
	);
	bool equal(
		Range<ASCIICharacter const>,
		ASCIICharacter const *
	);
	bool equal(
		ASCIICharacter const *,
		ASCIICharacter const *
	);
	bool equal(UTF16Unit const *, UTF16Unit const *);

	// Compares two zero-terminated strings for sorting, just like strcmp().
	//   return value is 
	//     zero if both strings are equal
	//     negative if the left string comes first
	//     positive if the right string comes first
	SIntLeast1B compare(ASCIICharacter const *, ASCIICharacter const *);
	SIntLeast1B compare(ASCIICharacter const *, UCount, ASCIICharacter const *, UCount);

	bool beginsWith(
		ASCIICharacter const *,
		ASCIICharacter const *
	);
	bool beginsWith(
		UTF16Unit const *     ,
		ASCIICharacter const *
	);

	// Checks whether the given text begins with the given literal.
	//   the literals terminating zero is ignored
	//   no byte-wise comparison, so the entire length of the literal must be accessable
	//   much faster than beginsWith()
	template <UCount length> bool beginsWithLiteral_lengthGuaranteed(
		ASCIICharacter const *  toText,
		ASCIICharacter const (& literal)[length]
	) {
		return equals(toText, literal, length - 1);
	}

	// Determines whether the given strings equal.
	// (complexity: linear in length of shorter string)
	bool equal(
		Range<UTF16Unit const> const & ,
		Range<UTF16Unit const> const &
	);

	// Determines whether the given strings equal without case sensitivity.
	// (complexity: linear in length of shorter string)
	bool equalCaseInsensitive(
		Range<ASCIICharacter const> const & ,
		Range<ASCIICharacter const> const &
	);

	// Determines whether the given string equals the given zero-terminated lowercase literal.
	// Used to detect keywords quickly.
	// (complexity: linear in length of literal)
	bool equalsCaseInsensitiveLiteral(
		ASCIICharacter const * toString,
		ASCIICharacter const * toStringsEnd,
		ASCIICharacter const * toZeroTerminatedLiteral
	);
	bool equalsCaseInsensitiveLiteral(
		Range<ASCIICharacter const> const & ,
		ASCIICharacter const *              toZeroTerminatedLiteral
	);
	bool equalsCaseInsensitiveLiteral(
		UTF16Unit const *      toZeroTerminatedString,
		ASCIICharacter const * toZeroTerminatedLiteral
	);

	// Detects the given keyword at the beginning of the given text.
	//   requires
	//     the entire literal being found at the beginning of the text (case sensitive)
	//     whitespace following or the text ending
	//   on match, returns yes and advances toText to just after the match
	//   otherwise, returns no and leaves toText untouched
	//   the literal must be zero-terminated
	bool parse(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toEnd,
		ASCIICharacter const *   toZeroTerminatedLiteral
	);

	// Detects the given keyword at the beginning of the given text.
	//   requires
	//     the entire literal being found at the beginning of the text (case insensitive)
	//     whitespace following or the text ending
	//   on match, returns yes and advances toText to just after the match
	//   otherwise, returns no and leaves toText untouched
	//   the literal must
	//     contain only lowercase characters
	//     be zero-terminated
	bool parse_caseInsensitive(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toEnd,
		ASCIICharacter const *   toZeroTerminatedLowerCaseLiteral
	);

	// Moves the given ranges beginning to the first non-whitespace character, or to the end. Any white space, control characters
	//  or line breaks are skipped.
	void skipWhitespace(Range<ASCIICharacter const> &);
	void skipWhitespace(ASCIICharacter const * & toText, ASCIICharacter const * toEnd);

	// Moves the given ranges beginning to the first non-whitespace character, line break, or to the end. Any white space or
	//  control characters are skipped.
	void skipWhitespaceButNotLineBreaks(Range<ASCIICharacter const> &);
	void skipWhitespaceButNotLineBreaks(ASCIICharacter const * & toText, ASCIICharacter const * toEnd);

	// Moves the given ranges beginning to behind the current line.
	// Useful for parsers to skip single-line comments.
	void skipLine(Range<ASCIICharacter const> &);
	void skipLine(ASCIICharacter const * & toText, ASCIICharacter const * toEnd);

	// Tries to read text enclosed in double quotes.
	//   Returns yes on success, moves toText to behind the quotation, and assigns the quotations beginning and end (without
	//    the double quotes).
	//   Returns no on failure.
	bool tryToReadQuotation(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toEndOfText,
		ASCIICharacter const * & toQuotation,
		ASCIICharacter const * & toQuotationsEnd
	);

	// Tries to read text enclosed in square brackets (common in INI files).
	//   Returns yes on success, moves toText to behind the name, and assigns the names beginning and end (without the
	//    square brackets).
	//   Returns no on failure.
	bool tryToReadINISectionName(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toEndOfText,
		ASCIICharacter const * & toName,
		ASCIICharacter const * & toNamesEnd
	);

	// Tries to read a property from an INI file.
	//   Returns yes on success. toText is moved to the end of the property. The name is assigned. If a value exists, it is
	//    assigned; otherwise, it is empty. The value is not filtered for whitespace.
	//   Returns no on failure. toText is moved to the point of failure; the other iterators are undefined.
	bool tryToReadINIProperty(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toTextsEnd,
		ASCIICharacter const * & toName,
		ASCIICharacter const * & toNamesEnd,
		ASCIICharacter const * & toValue,
		ASCIICharacter const * & toValuesEnd
	);

	// Counts the lines of text in the given range.
	// Useful for parsers: error messages typically require a line where the error occured. Instead of complicating the parser by
	//  tracking the current line, it will just track the current text position. Only if an error occurs, the number of lines from
	//  the beginning of the text to the current position is counted to print an error message.
	// (complexity: linear in length of text)
	UCount countLinesIn(Range<ASCIICharacter const> const & text);

	// Converts the given ASCII string to a UTF-16 string.
	void toUTF16(
		Range<ASCIICharacter const> const & source,
		Range<UTF16Unit> const &            destination
	);

	// Converts the given integer number to a string of decimal digits.
	//   the destination must be large enough to store the result
	//   no terminating zero
	//   returns the a pointer to the end of the written string
	ASCIICharacter * toDecimalString(
		ASCIICharacter destination[sizeof "255"],
		UInt1B         number
	);
	ASCIICharacter * toDecimalString(
		ASCIICharacter destination[sizeof "65536"],
		UInt2B         number
	);
	ASCIICharacter * toDecimalString(
		ASCIICharacter destination[sizeof "4294967295"],
		UInt4B         number
	);
	ASCIICharacter * toDecimalString(
		ASCIICharacter destination[sizeof "18446744073709551615"],
		UInt8B         number
	);
	ASCIICharacter * toDecimalString(
		ASCIICharacter destination[sizeof "-128"],
		SInt1B         number
	);
	ASCIICharacter * toDecimalString(
		ASCIICharacter destination[sizeof "-32768"],
		SInt2B         number
	);
	ASCIICharacter * toDecimalString(
		ASCIICharacter destination[sizeof "-2147483648"],
		SInt4B         number
	);
	ASCIICharacter * toDecimalString(
		ASCIICharacter destination[sizeof "-9223372036854775808"],
		SInt8B         number
	);
	UTF16Unit * toDecimalString(
		UTF16Unit destination[sizeof "255"],
		UInt1B    number
	);
	UTF16Unit * toDecimalString(
		UTF16Unit destination[sizeof "65536"],
		UInt2B    number
	);
	UTF16Unit * toDecimalString(
		UTF16Unit destination[sizeof "4294967295"],
		UInt4B    number
	);
	UTF16Unit * toDecimalString(
		UTF16Unit destination[sizeof "18446744073709551615"],
		UInt8B    number
	);
	UTF16Unit * toDecimalString(
		UTF16Unit destination[sizeof "-128"],
		SInt1B    number
	);
	UTF16Unit * toDecimalString(
		UTF16Unit destination[sizeof "-32768"],
		SInt2B    number
	);
	UTF16Unit * toDecimalString(
		UTF16Unit destination[sizeof "-2147483648"],
		SInt4B    number
	);
	UTF16Unit * toDecimalString(
		UTF16Unit destination[sizeof "-9223372036854775808"],
		SInt8B    number
	);

	// Convertes the given integer to a string of hexadecimal digits with zero padding. The destination provided must be large
	//  enough to store the result. There is no terminating zero; a pointer to the end of the written number is returned.
	ASCIICharacter * toHexadecimalZeroPaddedString(
		UInt2B         number,
		ASCIICharacter destination[4]
	);
	UTF16Unit * toHexadecimalZeroPaddedString(
		UInt2B    number,
		UTF16Unit destination[4]
	);
	UTF16Unit * toHexadecimalZeroPaddedString(
		UTF16Unit destination[8],
		UInt4B    number
	);
	UTF16Unit * toHexadecimalZeroPaddedString(
		UTF16Unit destination[16],
		UInt8B    number
	);

	// Converts the given floating-point number to a decimal string.
	//   inserts the given symbol as decimal point
	//   enforcePlus forces a + for positive number (no prefix otherwise)
	ASCIICharacter * toDecimalString(
		double         ,
		UIntLeast1B    precision,
		ASCIICharacter decimalPoint,
		bool           enforcePlus,
		ASCIICharacter destination[349]
	);

	// Formats the given string in the same way sprintf() would. Returns the range of characters that has been formatted.
	Range<ASCIICharacter> format(
		Range<ASCIICharacter> const & destination,
		ASCIICharacter const *        toFormat,
		...
	);
	Range<ASCIICharacter> format(
		Range<ASCIICharacter> const &     destination,
		ASCIICharacter const *            toFormat,
		struct VariableParameters const * toParameters
	);

	// Converts a decimal string at toText to a 8-B unsigned integer and advances toText to the first character that was no
	//  decimal digit. Returns zero if no number was found at toText. Does not skip whitespace.
	UInt8B uInt8BFromDecimal(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toEnd
	);
	UInt4B uInt4BFromDecimal(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toEnd
	);
	UInt2B uInt2BFromDecimal(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toEnd
	);
	UCount numberFromDecimal(
		ASCIICharacter const * & toText,
		ASCIICharacter const *   toEnd
	);

	UInt8B uInt8BFromDecimal(
		UTF16Unit const * & toText,
		UTF16Unit const *   toEnd
	);
	UInt4B uInt4BFromDecimal(
		UTF16Unit const * & toText,
		UTF16Unit const *   toEnd
	);
	UInt2B uInt2BFromDecimal(
		UTF16Unit const * & toText,
		UTF16Unit const *   toEnd
	);
	UCount numberFromDecimal(
		UTF16Unit const * & toText,
		UTF16Unit const *   toEnd
	);

	// Converts the given string range to an IEEE 754 binary64 floating-point number, or returns NaN if not successful. After the
	//  function returns, the given ranges beginning either equals its end the first character that was not recognized as part of
	//  the number.
	// The lexical format of a floating-point number is
	//    [sign] [digits] [.digits] [ {d|D|e|E}[sign]digits ]
	//  where
	//    sign is {+|-}; and
	//    digits is a string of decimal digits.
	double float8BFromDecimal(ASCIICharacter const * & toBeginning, ASCIICharacter const * toEnd);
	double float8BFromDecimal(UTF16Unit const * & toBeginning, UTF16Unit const * toEnd);
	double float8BFromDecimal(Range<ASCIICharacter const> &);


} // Text


// Writes the given UUID to the given string.
//   requires 38 characters + one terminating zero
//   format is {XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX}
//   returns a pointer to the terminating zero
UTF16Unit * asString(UUID const & uid, UTF16Unit result[39]);