// Visual C++ 2015 C-Runtime replacements of intrinsic functions.
//   cannot be compiled with global optimizations, therefore need their own module



#include "../framework.hpp"



#if !_DEBUG // debug builds are too complicated to replace


	// intrinsics  available because were obviously using Visual C++, otherwise this file wouldnt be used
	import {
		void __movsb(void *, void const *, USize);
		void __stosb(void *, Byte, UCount);
		void * C_CALL memcpy(void *, void const *, USize);
		int C_CALL memcmp(void const *, void const *, USize);
		void * C_CALL memset(void *, int, USize);
	}

	import_from(Kernel32) {
		void API_CALL RtlMoveMemory(void *, void const *, USize);
	} // Kernel32


	extern "C" {

		// Used by Visual C++ for default initialization of POD:
		void * C_CALL memmove(void * d, void const * s, USize n) {
			RtlMoveMemory(d, s, n);
			return d;
		}

#		pragma function(memcmp)
		int memcmp(void const * a, void const * b, USize n) {
			//  There is RtlCompareMemory() in Kernel32.dll, but it is no exported by the 32-bit Windows SDK and instead requires
			//   the Driver Development Kit for import from NtDll.dll. No option because of DDK dependency hassle.
			auto toA = reinterpret_cast<UInt1B const *>(a);
			auto toB = reinterpret_cast<UInt1B const *>(b);
			while(n --> 0) {
				if(*toA != *toB) {
					return int(*toA - *toB);
				}
				++toA;
				++toB;
			}
			return 0;
		}

		// Used by Visual C++ for large POD copies:
#		pragma function(memcpy)
		void * C_CALL memcpy(void * d, void const * s, USize n) {
			__movsb(d, s, n);
			return d;
		}

		// Used by Visual C++ for POD initialization:
#		pragma function(memset)
		void * C_CALL memset(void * ptr, int value, USize num) {
			__stosb(ptr, Byte(value), num);
			return ptr;
		}

		// Raises floating-point exceptions. Called after floating-point operations are emulated in software (e.g.
		//  double-to-long long-conversion, which is not natively supported on 32-bit x86).
		double _except1(UInt4B, int, double, double result, UInt4B, void *) {
			return result;
		}


	} // C


#endif // release builds only