; All Visual C++ 2015 C-Runtime replacements which can only be implemented in Assembly (x86-64 versions).
;   functions for stack manipulation, e.g. _alloca()
;   security checks which run in function prologues



_PAGESIZE_ equ 1000h

.code

; __chkstk() probes the stack in 4096 B intervals. Visual C++ inserts it on _alloca() or if a function uses more than 4096 B
;  of stack space. It guarantees sufficient stack commit size for random access to local variables.
;
; The function is called from the prologs of other functions, and therefore does not follow the x86-64 ABI. This makes an
;  assembler implementation necessary:
;    the only parameter (size to probe) is passed in rax
;    there may be no room to store local variables, so only rax, r10, and r11 can be used
__chkstk proc
		; copied from code compiled by Visual C++ 2015:
		sub      rsp, 10h
		mov      qword ptr [rsp], r10
		mov      qword ptr [rsp + 8], r11
		xor      r11, r11
		lea      r10, [rsp + 18h]
		sub      r10, rax
		cmovb    r10, r11
		mov      r11, qword ptr gs:[10h]
		cmp      r10, r11
		bnd jae  cs20
		and      r10w, not(_PAGESIZE_ - 1)
	cs10:
		lea      r11, [r11 - _PAGESIZE_]
		mov      byte ptr [r11], 0
		cmp      r10, r11
		bnd jne  cs10
	cs20:
		mov      r10, qword ptr [rsp]
		mov      r11, qword ptr [rsp + 8]
		add      rsp, 10h
		bnd ret
__chkstk endp

_alloca_probe = __chkstk

end