namespace Formats {


	namespace BMP {


		namespace Info { enum : UInt4B {

			// info: non-zero padding between headers and image data (some programs store additional information there)
			//  (more)   address of padding
			//  (return) yes to ignore; no to abort
			imagePadding,

			// info: extra data after end of file
			//  (more)   to end of file
			//  (return) ignored
			extraData

		}; } // Info


		namespace Warning { enum : UInt4B {

			// The image size in the file differs from the expected image size. The file may be corrupted.
			badImageSize = 0x10000100,

			// warning: non-planar BMP
			//  (more)   nothing
			//  (return) yes to treat as planar anyway; no to abort
			nonPlanar,

			// warning: too many palette colors
			//  (more)   nothing
			//  (return) yes to ignore additional colors; no to abort
			paletteTooLong,

			// warning: image offset in header does not fit file layout
			//  (more)   nothing
			//  (return) yes to ignore; no to abort
			badImageOffset,

			// warning: unknown image color space
			//  (more)   nothing
			//  (return) yes to ignore; no to abort
			badColorSpace,

			// warning: unknown rendering intent
			//  (more)   nothing
			//  (return) yes to ignore; no to abort
			badRenderingIntent,

			// warning: color profile data should be embedded in the file, but the offset is invalid
			//  (more)   nothing
			//  (return) yes to ignore; no to abort
			badEmbeddedColorProfileOffset,

			// warning: color profile data should be embedded in the file, but the offset is invalid
			//  (more)   nothing
			//  (return) yes to ignore; no to abort
			badEmbeddedColorProfileSize,

			// warning: reserved field is not zero
			//  (more)   nothing
			//  (return) yes to ignore; no to abort
			nonZeroReserved,

			// warning: RLE files must be top-down
			//  (more)   nothing
			//  (return) yes to load anyway (pixels may be mirrored vertically); no to abort
			mirroredRLE,

			// warning: too many pixels
			//  (more)   pointer to RLE data which caused the overrun
			//  (return) yes to ignore additional pixels; no to abort
			rleOverrun,

			// warning: unexpected end of pixels
			//  (more)   pointer to end of RLE data
			//  (return) yes to pad missing pixels with transparency; no to abort
			eofInRLE,

			// warning: damaged CCITT T.4 stream: too many pixels in a line
			//  (more)   nothing
			//  (return) yes to ignore the additional pixels; no to abort
			ccittLineOverrun,

			// warning: damaged CCITT T.4 stream: too many lines
			//  (more)   nothing
			//  (return) yes to ignore the additional lines; no to abort
			ccittOverrun,

			// warning: unexpected end of CCITT T.4 stream
			//  (more)   nothing
			//  (return) yes to pad the rest of the image white; no to abort
			eofInCCITT

		}; } // Warning


		namespace Error { enum : UInt4B {
			// Unexpected end of file:
			eofInFileHeader = 0x80000100,
			eofInHeader,
			eofInColorMasks,
			eofInPalette,
			eofInPixels,
			// Invalid file data:
			badSignature,
			badVersion,
			badColorDepth,
			badHeight,

			// error: icons and cursors must have even heights (accounting for an implicit bit mask), but an odd height was found
			//  (more)   nothing
			//  (return) ignored
			badHeightForIcon,

			badImageSize,
			badCompression,
			// The BMP version is not yet supported.
			unsupportedVersion,
			// There is no data in the file.
			emptyImage,

			// error: the image is too large to be loaded
			//  (more)   nothing
			//  (return) ignored
			tooLarge,

			// error: the compression method is not yet supported (Huffman 1D, JPEG, PNG)
			//  (more)   nothing
			//  (return) ignored
			unsupportedCompression

		}; }


	} // BMP


} // Formats