namespace Formats {


	namespace IFF {


		namespace Info { enum {

			// ANNO encountered
			// (more)   to zero-terminated text (ANSI)
			// (return) ignored
			annotation = 0x10000400,

			// AUTH encountered
			// (more)   to zero-terminated text (ANSI)
			// (return) ignored
			author,

			// (c)  encountered
			// (more)   to zero-terminated text (ANSI)
			// (return) ignored
			copyright,

			// NAME encountered
			// (more)   to zero-terminated text (ANSI)
			// (return) ignored
			title

		}; } // Info


		namespace Warning { enum : UInt4B {

			// warning: damaged header (unexpected size)
			// (more)   to said IFF::Header
			// (return) yes to proceed to actual end of file; no to abort
			badSize = 0x40000400,

			// warning: unknown chunk
			// (more)   to said IFF::Chunk
			// (return) yes to skip said chunk and proceed; no to abort
			unknownChunk,

			// warning: end of file in chunk
			// (more)   to said IFF::Chunk
			// (return) yes to process the file without reading any farther; no to abort
			eofInChunk,

			// warning: damaged ANNO (text not zero-terminated)
			// (more)   to said text as ANSI string
			// (return) ignored
			anno_damaged,

			// warning: damaged AUTH (text not zero-terminated)
			// (more)   to said text as ANSI string
			// (return) ignored
			auth_damaged,

			// warning: damaged (c)  (text not zero-terminated)
			// (more)   to said text as ANSI string
			// (return) ignored
			copyright_damaged,

			// warning: damaged NAME (text not zero-terminated)
			// (more)   to said text as ANSI string
			// (return) ignored
			name_damaged

		}; } // Warning


		namespace Error { enum : UInt4B {

			// error: unexpected end of file in header
			//  (more)   to beginning of file
			//  (return) ignored
			eofInHeader = 0x80000400,

			// error: signature not FORM
			//  (more)   to beginning of file
			//  (return) ignored
			badSignature,

			// error: unknown media type
			//  (more)   to beginning of file
			//  (return) ignored
			badMediaType

		}; } // Error


		namespace BM {


			namespace Info { enum : UInt4B {

				// SPRT encountered; first color will be considered transparent
				// (more)   to ILBM::SPRT
				// (return) ignored
				sprt = 0x10000500

			}; } // Info


			namespace Warning { enum : UInt4B {

				// warning: damaged BMHD (larger than expected)
				// (more)   to said ILBM::BMHD
				// (return) yes to skip extra data and proceed; no to abort
				bmhd_tooLarge = 0x40000500,

				// warning: damaged BMHD (pixels but no bit depth)
				// (more)   to ILBM::BMHD
				// (return) yes to treat as palette-only file; no to abort
				bmhd_pixelsInPaletteOnly,

				// warning: damaged BMHD (unexpected masking)
				// (more)   to ILBM::BMHD
				// (return) yes to ignore masking; no to abort
				bmhd_badMasking,

				// warning: damaged BMHD (color key exceeds bit depth)
				// (more)   to end of file
				// (return) yes to ignore color key; no to abort
				bmhd_badColorKey,

				// warning: damaged CAMG (HAM flag set but not 58 bits per pixel)
				// (more)   to end of file
				// (return) yes to ignore Hold-and-Modify flag; no to abort
				camg_badHAM,

				// warning: damaged CAMG (EHB flag set but not 6 bits per pixel)
				// (more)   to end of file
				// (return) yes to ignore Extra Half-Brite flag; no to abort
				camg_badEHB,

				// warning: damaged CAMG (unexpected size)
				// (more)   to ILBM::BMHD
				// (return) yes to skip; no to abort
				camg_badSize,

				// warning: damaged CAMG (missing HAM flag)
				// (more)   to ILBM::CAMG
				// (return) yes to ignore; no to abort
				camg_noHAM,

				// warning: damaged CAMG (missing EHB flag)
				// (more)   to ILBM::CAMG
				// (return) yes to ignore; no to abort
				camg_noEHB,

				// warning: damaged CMAP (too few colors)
				// (more)   to ILBM::CMAP
				// (return) yes to default missing colors to VGA palette; no to abort
				cmap_tooFew,

				// warning: damaged CMAP (too many colors)
				// (more)   to ILBM::CMAP
				// (return) yes to proceed; no to abort
				cmap_tooMany,

				// warning: damaged CMAP (HAM and EHB flags conflict)
				// (more)   to ILBM::CMAP
				// (return) yes to ignore one of the flags; no to abort
				cmap_HAMconflictsEHB,

				// warning: stray CMAP in true-color image
				// (more)   to ILBM::CMAP
				// (return) yes to skip; no to abort
				cmap_withTrueColor,

				// warning: CMYK is not yet supported
				// (more)   to IFF::Chunk
				// (return) yes to skip; no to abort
				cmyk_skipping,

				// warning: CRNG (Deluxe Paint Color Cycle) is not yet supported
				// (more)   to IFF::Chunk
				// (return) yes to skip; no to abort
				crng_skipping,

				// warning: CTBL must appear after BMHD; skipping
				// (more)   to ILBM::CTBL
				// (return) ignored
				ctbl_beforeBMHD,

				// warning: damaged CTBL (too small); skipping
				// (more)   to ILBM::CTBL
				// (return) ignored
				ctbl_tooSmall,

				// warning: CTBL (per-line color palettes) must be unique but appears twice
				// (more)   to ILBM::CTBL
				// (return) yes to skip; no to abort
				ctbl_duplicate,

				// warning: damaged CTBL (too large)
				// (more)   to ILBM::CTBL
				// (return) yes to load anyway; no to abort
				ctbl_tooLarge,

				// warning: DEST is not yet supported
				// (more)   to IFF::Chunk
				// (return) yes to skip; no to abort
				dest_skipping,

				// warning: DPI  (physical resolution) must be unique but appears twice
				// (more)   to ILBM::DPI
				// (return) yes to skip; no to abort
				dpi_duplicate,

				// warning: damaged DPI  (unexpected size)
				// (more)   to ILBM::DPI
				// (return) yes to skip; no to abort
				dpi_badSize,

				// warning: DPPV (Deluxe Paint II perspective) is not yet supported
				// (more)   to IFF::Chunk
				// (return) yes to skip; no to abort
				dppv_skipping,

				// warning: DPPS (Deluxe Paint III perspective) is not yet supported
				// (more)   to IFF::Chunk
				// (return) yes to skip; no to abort
				dpps_skipping,

				// warning: DRNG (Deluxe Paint IV Enhanced Color Cycle) is not yet supported
				// (more)   to IFF::Chunk
				// (return) yes to skip; no to abort
				drng_skipping,

				// warning: GRAB (hotspot for brushes and cursors) is not yet supported
				// (more)   to IFF::Chunk
				// (return) ignored
				grab_skipping,

				// warning: SHAM must appear after BMHD; skipping
				// (more)   to ILBM::SHAM
				// (return) ignored
				sham_beforeBMHD,

				// warning: damaged SHAM (too small); skipping
				// (more)   to ILBM::SHAM
				// (return) ignored
				sham_tooSmall,

				// warning: SHAM (per-line color palettes) must be unique but appears twice
				// (more)   to ILBM::SHAM
				// (return) yes to skip; no to abort
				sham_duplicate,

				// warning: damaged SHAM (too large)
				// (more)   to ILBM::SHAM
				// (return) yes to load anyway; no to abort
				sham_tooLarge,

				// warning: damaged SHAM (version not zero)
				// (more)   to ILBM::SHAM
				// (return) yes to load anyway; no to abort
				sham_badVersion,

				// warning: SPRT must appear after BMHD
				// (more)   to ILBM::SPRT
				// (return) yes to skip; no to abort
				sprt_beforeBMHD,

				// warning: TINY (Deluxe Paint III thumbnail) must be unique but appears twice
				// (more)   to ILBM::TINY
				// (return) yes to skip; no to abort
				tiny_duplicate,

				// warning: damaged TINY (unexpected size)
				// (more)   to ILBM::TINY
				// (return) yes to skip; no to abort
				tiny_badSize,

				// warning: damaged TINY (hidden data)
				// (more)   to ILBM::TINY
				// (return) yes to extract hidden data; no to abort
				tiny_hidden,

				// warning: damaged TINY (empty)
				// (more)   to ILBM::TINY
				// (return) yes to skip; no to abort
				tiny_empty

			}; } // Warning


			namespace Error { enum : UInt4B {

				// error: BMHD missing
				//  (more)   to end of file
				//  (return) ignored
				bmhd_missing = 0x80000500,

				// error: BMHD must be unique but appears twice
				//  (more)   to IFF::Chunk containing said ILBM::BMHD
				//  (return) ignored
				bmhd_duplicate,

				// error: BMHD too small
				//  (more)   to IFF::Chunk containing said ILBM::BMHD
				//  (return) ignored
				bmhd_tooSmall,

				// error: image too wide (>4096)
				//  (more)   to IFF::Chunk containing ILBM::BMHD
				//  (return) ignored
				bmhd_tooWide,

				// error: image too high (>4096)
				//  (more)   to IFF::Chunk containing ILBM::BMHD
				//  (return) ignored
				bmhd_tooHigh,

				// error: compressed ACBMs not supported
				//  (more)   to end of file
				//  (return) ignored
				compressedACBMNotSupported,

				// error: no image dimension can be zero
				//  (more)   to IFF::Chunk containing ILBM::BMHD
				//  (return) ignored
				bmhd_emptyImage,

				// error: unknown compression
				//  (more)   to IFF::Chunk containing ILBM::BMHD
				//  (return) ignored
				bmhd_badCompression,

				// error: unknown bit depth (only 07 & 24 expected)
				//  (more)   to IFF::Chunk containing ILBM::BMHD
				//  (return) ignored
				bmhd_badBitDepth,

				// error: BODY missing
				//  (more)   to end of file
				//  (return) ignored
				body_missing,

				// error: BODY must be unique but appears twice
				//  (more)   to IFF::Chunk containing BODY
				//  (return) ignored
				body_duplicate,

				// error: CMAP missing
				//  (more)   to end of file
				//  (return) ignored
				cmap_missing,

				// error: CMAP must be unique but appears twice
				//  (more)   to IFF::Chunk containing CMAP
				//  (return) ignored
				cmap_duplicate,

				// error: CMAP does not have enough colors for bit depth
				//  (more)   to end of file
				//  (return) ignored
				cmap_tooFew

			}; } // Error


		} // BM


	} // IFF


} // Formats