namespace Formats {


	namespace PCX {


		namespace Warning { enum : UInt4B {

			// warning: bad header (plane size odd or too small)
			// (more)   nothing
			// (return) yes to proceed with a better plane size; no to abort
			badPlaneSize = 0x40000600,

			// warning: bad header (unknown palette flags)
			// (more)   nothing
			// (return) yes to ignore; no to abort
			badPaletteInfo,

			// warning: non-zero padding in header (some programs might refuse to load the image)
			// (more)   to 54 B of padding
			// (return) yes to ignore; no to abort
			nonZeroPaddingInHeader,

			// warning: bad pixels (unexpected end of file)
			// (more)   nothing
			// (return) yes to ignore; no to abort
			eofInPixels,

			// warning: bad pixels (compression overrun)
			// (more)   nothing
			// (return) yes to ignore; no to abort
			excessPixels,

			// warning: non-zero padding in palette
			// (more)   to 48 B of padding
			// (return) yes to ignore; no to abort
			nonZeroPaddingInPalette,

			// warning: no palette at expected position
			// (more)   nothing
			// (return) yes to load from the expected position; no to abort
			unexpectedPalette,

			// warning: bad palette (does not begin with 12)
			// (more)   nothing
			// (return) yes to load anyway; no to abort
			badPalette,

			// warning: bad palette (all colors equal)
			// (more)   nothing
			// (return) no to abort; yes to change
			//           second color to opposite of first (1-bit images)
			//           all colors to default EGA palette (4-bit images)
			monochromePalette,

			// warning: extra data after the end of the file
			// (more)   nothing
			// (return) yes to ignore; no to abort
			extraData

		}; }


		namespace Error { enum : UInt4B {

			// error: bad header (unexpected end of file)
			// (more)   nothing
			// (return) ignored
			eofInHeader = 0x80000600,

			// error: bad header (signature not 10)
			// (more)   nothing
			// (return) ignored
			badSignature,

			// error: bad header (version not 0, 2, 3, 4, or 5)
			// (more)   nothing
			// (return) ignored
			badVersion,

			// error: bad header (compression not 0 or 1)
			// (more)   nothing
			// (return) ignored
			badCompression,

			// error: bad header (bits per pixel not 1, 2, 4, or 8)
			// (more)   nothing
			// (return) ignored
			badBitsPerPixel,

			// error: bad header (image dimensions negative or zero)
			// (more)   nothing
			// (return) ignored
			badDimensions,

			// error: bad header (number of planes not 1, 2, 3, or 4)
			// (more)   nothing
			// (return) ignored
			badNumberOfPlanes,

			// error: bad palette (unexpected end of file)
			// (more)   nothing
			// (return) ignored
			eofInPalette

		}; } // Error


	} // PCX


} // Formats