namespace Formats {


	namespace STL {


		namespace Info { enum : UInt4B {

			// Indicates which program is most likely to have created the file.
			// (more)   zero-terminated ASCII string containing program name
			// (return) ignored
			flavor = 0x10000800,

			// Reports the creation date which was stored in the file.
			// (more)   Formats::FileString containing the vendor-specific date, e.g.
			//             Cura: Thu 28 Jul 2016 04:39:55
			//             Flashforge: 20160805 01:41:42
			//             Rhino: Dec  1 2014
			// (return) ignored
			writeDate,

			// Reports the path where the STL was originally stored.
			// (more)   Formats::FileString containing the path
			// (return) ignored
			originalPath

		}; }


		namespace Warning { enum : UInt4B {

			// No metadata found in the file. It may be scaled, colored, or oriented incorrectly.
			// (more)   nothing
			// (return) ignored
			unknownFlavor = 0x40000800,

			// Netfabb metadata is embedded in the file, but its unit is unknown. The file may be scaled incorrectly.
			// (more)   nothing
			// (return) ignored
			badNetfabbUnit,

			// ArchiCAD metadata is embedded in the file, but its unit is unknown. The file may be scaled incorrectly.
			// (more)   nothing
			// (return) ignored
			badArchiCADUnit,

			// ArchiCAD metadata is embedded in the file, but its scale is unknown. The file may be scaled incorrectly.
			// (more)   nothing
			// (return) ignored
			badArchiCADScale,

			// Non-standard STL: missing name after endsolid.
			// (more)   file offset (in bytes)
			// (return) yes to ignore; no to abort
			badEndsolid,

			// Non-standard STL: end solid instead of endsolid. This is an IRONCAD oddity.
			// (more)   file offset (in bytes)
			// (return) yes to ignore; no to abort
			badEnd_solid,

			// Unexpected end of file.
			// (more)   nothing
			// (return) yes to ignore; no to abort
			unexpectedEndOfFile

		}; } // Warning


		namespace Error { enum : UInt4B {

			// error: expected solid keyword
			// (more)   offset in file, in B
			// (return) ignored
			expected_solid = 0x80000800,

			// error: expected name of solid
			// (more)   offset in file, in B
			// (return) ignored
			expectedSolidName,

			// error: expected either facet or endsolid keyword
			// (more)   offset in file, in B
			// (return) ignored
			expected_facet_endsolid,

			// error: expected vertex keyword
			// (more)   offset in file, in B
			// (return) ignored
			expected_vertex,

			// error: expected a number
			// (more)   offset in file, in B
			// (return) ignored
			expectedNumber,

			// error: expected endloop keyword
			// (more)   offset in file, in B
			// (return) ignored
			expected_endloop,

			// error: expected endfacet keyword
			// (more)   offset in file, in B
			// (return) ignored
			expected_endfacet,

			// error: unexpected end of file in number of triangles
			// (more)   nothing
			// (return) ignored
			eofInTriangleCount,

			// error: unexpected end of file in triangles
			// (more)   nothing
			// (return) ignored
			eofInTriangles

		}; } // Error


	} // STL


} // Formats