namespace Formats {


	namespace VRML {


		namespace Info { enum : UInt4B {

			// Indicates which program is most likely to have created the file.
			// (more)   zero-terminated ASCII string containing program name
			// (return) ignored
			flavor = 0x10000880,

			// Reports the creation date which was stored in the file.
			// (more)   Formats::FileString containing the vendor-specific date, e.g.
			//             3ds Max: Mon Jun 24 08:59:46 2013
			//             Tr3: 20040505
			// (return) ignored
			writeDate,

			// Reports the path where the VRML file was originally stored.
			// (more)   Formats::FileString containing the path
			// (return) ignored
			originalPath,

			// Reports the path of the original file from which VRML was exported.
			// (more)   Formats::FileString containing the path
			// (return) ignored
			sourcePath

		}; }


		namespace Warning { enum : UInt4B {

			// found a non-conforming OpenInventor extension
			// (more)   nothing
			// (return) yes to ignore; no to abort
			v1_unsupportedExtension_OpenInventor = 0x40000880,

			// found a non-conforming Live3D extension
			// (more)   nothing
			// (return) yes to ignore; no to abort
			v1_unsupportedExtension_Live3D,

			// found a non-conforming VRWeb extension
			// (more)   nothing
			// (return) yes to ignore; no to abort
			v1_unsupportedExtension_VRWeb,

			// 44 matrices are not supported
			// (more)   nothing
			// (return) yes to ignore the last column; no to abort
			v1_unsupported_4x4,

			// Switch node choice -3 is deprecated
			// (more)   nothing
			// (return) yes to proceed; no to abort
			v1_Switch_whichChoice_deprecated,

			// the scenes background color has been defined more than once
			// (more)   nothing
			// (return) yes to ignore; no to abort
			v1_backgroundColor_duplicate,

			// no URL in ImageTexture
			// (more)   nothing
			// (return) yes to read anyway; no to abort
			v2_ImageTexture_noURL,

			// more than one URL in ImageTexture
			// (more)   nothing
			// (return) yes to read anyway; no to abort
			v2_ImageTexture_tooManyURLs,

			// MovieTexture and PixelTexture nodes are not yet supported
			// (more)   nothing
			// (return) yes to ignore; no to abort
			v2_notYetImplemented_MovieTexturePixelTexture,

			// IndexedFaceSet.coord is NULL
			// (more)   nothing
			// (return) yes to ignore; no to abort
			v2_IndexedFaceSet_coord_empty,

			// there is more than one WorldInfo node
			// (more)   nothing
			// (return) yes to ignore the duplicate; no to abort
			v2_WorldInfo_duplicate,

			// EXTERNPROTO not found
			// (more)   nothing
			// (return) yes to ignore all instances; no to abort
			v2_EXTERNPROTO_unresolved,

			// PROTO contains more than one node
			// (more)   nothing
			// (return) yes to ignore the additional nodes; no to abort
			v2_PROTO_tooManyNodes,

			// invalid vector
			// (more)   nothing
			// (return) yes to assume zero; no to abort
			badVector,

			// invalid color (negative or way too bright)
			// (more)   nothing
			// (return) yes to assume white; no to abort
			badColor,

			// invalid color (too bright)
			// (more)   nothing
			// (return) yes to darken; no to abort
			badColor_tooBright,

			// invalid polygon (has just one or two vertices)
			// (more)   nothing
			// (return) yes to ignore; no to abort
			degeneratedPolygon,

		}; } // Warning


		namespace Error { enum : UInt4B {

			// unexpected end of file
			// (more)   nothing
			// (return) ignored
			v1_endOfFile = 0x80000880,

			// missing {
			// (more)   nothing
			// (return) ignored
			v1_syntax_missingOpeningBrace,

			// a node name begins with a digit, which is forbidden
			// (more)   nothing
			// (return) ignored
			v1_syntax_digitInNodeName,

			// a name contains a control character, which is forbidden
			// (more)   nothing
			// (return) yes to proceed; no to abort
			v1_syntax_controlCharacterInName,

			// a name contains a special character (single quote, double quote, backslash, plus, or dot), which is forbidden
			// (more)   nothing
			// (return) yes to proceed; no to abort
			v1_syntax_specialCharacterInName,

			// field names must begin with lowercase letters
			// (more)   nothing
			// (return) ignored
			v1_syntax_firstFieldNameLetterNotLowercase,

			// node names must begin with uppercase letters
			// (more)   nothing
			// (return) ignored
			v1_syntax_firstNodeNameLetterNotUppercase,

			// unknown field type
			// (more)   nothing
			// (return) ignored
			v1_syntax_badFieldType,

			// missing a number
			// (more)   nothing
			// (return) ignored
			v1_syntax_missingNumber,

			// missing a string literal
			// (more)   nothing
			// (return) ignored
			v1_syntax_missingString,

			// a field was encountered after child nodes, which is forbidden
			// (more)   nothing
			// (return) yes to read anyway; no to abort
			v1_syntax_fieldOrder,

			// missing the fields keyword
			// (more)   nothing
			// (return) ignored
			v1_syntax_missingFields,

			// missing a node or field definition
			// (more)   nothing
			// (return) ignored
			v1_syntax_missingDefinition,

			// bad choice in Switch node
			// (more)   nothing
			// (return) yes to ignore all children; no to abort
			v1_Switch_whichChoice_bad,

			// bad vertexOrdering in ShapeHints
			// (more)   nothing
			// (return) yes to assume counterclockwise; no to abort
			v1_ShapeHints_vertexOrdering_bad,

			// bad shapeType in ShapeHints
			// (more)   nothing
			// (return) yes to assume unknown; no to abort
			v1_ShapeHints_shapeType_bad,

			// boolean value not FALSE, TRUE, 0, or 1
			// (more)   nothing
			// (return) ignored
			v1_bad_SFBool,

			// vertex, normal, or UV binding unknown
			// (more)   nothing
			// (return) ignored
			v1_bad_binding,

			// a bitfield exceeds the maximal length of 32 elements
			// (more)   nothing
			// (return) ignored
			v1_bitfieldTooLong,

			// a custom node type has more than 255 fields
			// (more)   nothing
			// (return) ignored
			v1_tooManyFields,

			// bad parts in Cylinder
			// (more)   nothing
			// (return) yes to assume none; no to abort
			v1_bad_Cylinder_parts,

			// bad parts in Cone
			// (more)   nothing
			// (return) yes to assume none; no to abort
			v1_bad_Cone_parts,

			// unexpected end of file
			// (more)   nothing
			// (return) ignored
			v2_syntax_endOfFile,

			// error in string literal
			// (more)   nothing
			// (return) ignored
			v2_syntax_badLiteral,

			// an identifier exceeds 65535 characters
			// (more)   nothing
			// (return) ignored
			v2_syntax_identifierTooLong,

			// whitespace exceeds 65535 consecutive characters
			// (more)   nothing
			// (return) ignored
			v2_syntax_whitespaceTooLong,

			// missing eventIn, eventOut, field, or exposedField
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingFieldRole,

			// missing field type
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingFieldType,

			// missing an identifier
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingIdentifier,

			// an identifier starts with a digit, which is forbidden
			// (more)   nothing
			// (return) yes to read anyway; no to abort
			v2_syntax_identifierStartsWithDigit,

			// a node or field name exceeds the 50-octets limit
			// (more)   nothing
			// (return) yes to read anyway; no to abort
			v2_syntax_nameTooLong,

			// missing {
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingOpeningBrace,

			// missing TRUE or FALSE
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingBool,

			// missing an integer literal
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingInteger,

			// missing a floating-point literal
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingFloat,

			// missing a string literal
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingString,

			// missing TO in ROUTE
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingTO,

			// missing node definition, ROUTE, or prototype definition
			// (more)   nothing
			// (return) ignored
			v2_syntax_missingStatement,

			// missing prototype name
			// (more)   nothing
			// (return) ignored
			v2_badPROTO,

			// expected a node of different type here
			// (more)   nothing
			// (return) ignored
			v2_wrongNodeType, // TODO: always treated as NULL; no way to opt-out

			// an axis is not normalized, which is forbidden
			// (more)   nothing
			// (return) yes to normalize; no to abort
			v2_axisNotNormalized,

			// Script nodes cannot use exposedField
			// (more)   nothing
			// (return) yes to ignore; no to abort
			v2_exposedFieldInScript,

			// a field does not exist
			// (more)   nothing
			// (return) ignored
			v2_badField,

			// a field has already been defined
			// (more)   nothing
			// (return) yes to ignore the second definition; no to abort
			v2_multiplyDefinedField,

			// cannot use IS outside of PROTO
			// (more)   nothing
			// (return) ignored
			v2_ISoutsidePROTO,

			// a parameter of that name does not exist
			// (more)   nothing
			// (return) ignored
			v2_badISName,

			// wrong type for this parameter
			// (more)   nothing
			// (return) ignored
			v2_badISType,

			// a PROTO contains no nodes, which is forbidden
			// (more)   nothing
			// (return) ignored
			v2_PROTO_empty,

			// nodes are nested too deep
			// (more)   nothing
			// (return) ignored
			nestingTooDeep,

			// USE on a value that has not been DEFined
			// (more)   nothing
			// (return) ignored
			bad_USE,

			// a rotation axis is zero or infinite
			// (more)   nothing
			// (return) yes to assume Z axis; no to abort
			bad_axis,

			// a rotation angle is infinite
			// (more)   nothing
			// (return) yes to assume zero; no to abort
			bad_angle,

			// an array exceeds the maximal length of 2^31 elements
			// (more)   nothing
			// (return) ignored
			arrayTooLong,

			// an image has no width or it exceeds the maximal width of 65535 pixels
			// (more)   nothing
			// (return) ignored
			bad_imageWidth,

			// an image has no height or it exceeds the maximal height of 65535 pixels
			// (more)   nothing
			// (return) ignored
			bad_imageHeight,

			// an image does not have the expected 1, 2, 3, or 4 components
			// (more)   nothing
			// (return) ignored
			bad_imageComponents,

			// normals are PER_FACE but seem like PER_VERTEX_INDEXED
			// (more)   nothing
			// (return) yes to assume PER_VERTEX_INDEXED; no to abort
			normals_perFace_are_perVertexIndexed,

			// normals are PER_VERTEX_INDEXED but seem like PER_VERTEX
			// (more)   nothing
			// (return) yes to assume PER_VERTEX; no to abort
			normals_perVertexIndexed_are_perVertex,

			// materials are PER_FACE but seem like PER_VERTEX_INDEXED
			// (more)   nothing
			// (return) yes to assume PER_VERTEX_INDEXED; no to abort
			materials_perFace_are_perVertexIndexed,

			// materials are PER_VERTEX_INDEXED but seem like PER_VERTEX
			// (more)   nothing
			// (return) yes to assume PER_VERTEX; no to abort
			materials_perVertexIndexed_are_perVertex,

			// insufficient normals indices
			// (more)   nothing
			// (return) yes to assume zero normals; no to abort
			missingNormals,

			// insufficient material indices
			// (more)   nothing
			// (return) yes to assume default material; no to abort
			missingMaterials,

			// insufficient UV indices
			// (more)   nothing
			// (return) yes to assume zero coordinates; no to abort
			missingUVs,

		}; } // Error


	} // VRML


} // Formats