namespace OS {


	namespace Win32 {


		namespace D3D9 {


			namespace Info { enum : UInt4B {

				// reports the D3D SDK version
				//  (more)   SDK version (UInt1B)
				//  (return) ignored
				sdkVersion = 0x10000C00,

				// selected n Anti-Aliasing
				//  (more)   AA (UInt1B)
				//  (return) ignored
				aaSelected,

				// whether a windowed context will be created
				//  (more)   zero if full-screen, non-zero otherwise
				//  (return) ignored
				windowed,

				// whether the GPU supports hardware Transform & Lighting
				//  (more)   zero if software TnL, non-zero otherwise
				//  (return) ignored
				tnl,

				// selected n anisotropic filtering
				//  (more)   filter level (1 for trilinear)
				//  (return) ignored
				afSelected

			}; } // Info


			namespace Warning { enum : UInt4B {

				// desired anti-aliasing is not available
				//  (more)   nothing
				//  (return) ignored
				noAA = 0x40000C00,

				// the GPU does not support vertical synchronization; tearing might occur
				//  (more)   nothing
				//  (return) ignored
				tearing,

				// the GPU cannot auto-generate mip maps; textures might look ugly
				//  (more)   nothing
				//  (return) ignored
				noMipMaps,

				// the GPU does not support linear color space rendering
				//  (more)   nothing
				//  (return) ignored
				noSRGB,

				// the GPU does not support alpha testing
				//  (more)   nothing
				//  (return) ignored
				noAlphaTesting,

				// The GPU context has been lost, most likely due to minimization/occlusion. Rendering is suspended.
				//  (more)   D3D9 error code
				//  (return) ignored
				lostGPU

			}; } // Warning


			namespace Error { enum : UInt4B {

				// Direct3D 9 is not installed
				//  (more)   nothing
				//  (return) ignored
				noD3D9 = 0x80000C00,

				// Direct3D 9Ex is not installed
				//  (more)   nothing
				//  (return) ignored
				noD3D9Ex,

				// there is no hardware-accelerated, D3D9-compatible GPU available
				//  (more)   nothing
				//  (return) ignored
				noHardwareAcceleration,

				// the GPU does not support shader model 2.0
				//  (more)   nothing
				//  (return) ignored
				noShaderModel2,

				// the GPU does not support the depth buffer format
				//  (more)   nothing
				//  (return) ignored
				noDepthBufferSupport,

				// the GPU does not support the minimal set of texture formats (A8R8G8B8, L8)
				//  (more)   nothing
				//  (return) ignored
				noTextureSupport,

				// unspecified error in the driver or in D3D
				//  (more)   D3D9 error code
				//  (return) ignored
				internal,

				// the API has been called with invalid parameters, in an invalid state, or the functionality is not available (most
				//  likely a programming error)
				//  (more)   D3D9 error code
				//  (return) ignored
				misuse,

				// D3D 9 does not support the operation
				//  (more)   D3D9 error code
				//  (return) ignored
				notSupported,

				// the GPU does not support the operation
				//  (more)   D3D9 error code
				//  (return) ignored
				notSupportedByGPU,

				// the GPU hung; this may be a driver problem
				//  (more)   D3D9 error code
				//  (return) ignored
				gpuHung,

				// the GPU has been disconnected
				//  (more)   D3D9 error code
				//  (return) ignored
				gpuRemoved

			}; } // Error


		} // D3D9


	} // Win32


} // OS