namespace OS {


	namespace Win32 {


		namespace RawInput {


			namespace Info { enum : UInt4B {

				// no HIDs connected
				//  (more)   nothing
				//  (return) ignored
				noHIDs = 0x10000E00,

				// HIDs connected, but none of them is suitable
				//  (more)   nothing
				//  (return) ignored
				noSuitableHIDs,

				// successfully opened a HID for reading
				//  (more)   zero-terminated name (UTF-16)
				//  (return) ignored
				hidOpen,

				// mimicking DirectInput, this HIDs slider is automatically remapped to Z
				//  (more)   nothing
				//  (return) ignored
				sliderIsZ,

				// mimicking DirectInput, the given axis is overridden
				//  (more)   index of axis
				//  (return) ignored
				overridingAxis,

				// mimicking DirectInput, the given button is overridden
				//  (more)   index of button
				//  (return) ignored
				overridingButton,

				// information on an axis
				//  (more)   address of an InfoOnAxis structure
				//  (return) ignored
				onAxis,

				// information on a button
				//  (more)   address of an InfoOnButton structure
				//  (return) ignored
				onButton,

				// a preferred HID was selected (according to DirectInput)
				//  (more)   zero-terminated name (UTF-16)
				//  (return) ignored
				preferredHID,

				// the preferred HID is connected
				//  (more)   zero-terminated name (UTF-16)
				//  (return) ignored
				preferredHIDConnected,

				// the preferred HID is not connected
				//  (more)   nothing
				//  (return) ignored
				preferredHIDNotConnected

			}; } // Info

			// Data of the Info::onAxis event.
			struct InfoOnAxis {
				UInt2B            index;
				UsageID           id;
				SInt4B            logicalCalibratedMinimum;
				SInt4B            logicalCenterMinimum;
				SInt4B            logicalCenterMaximum;
				SInt4B            logicalCalibratedMaximum;
				UTF16Unit const * name;
			};

			// Data of the Info::onButton event.
			struct InfoOnButton {
				UInt2B            index;
				UsageID           id;
				UTF16Unit const * name;
			};


			namespace Error { enum : UInt4B {

				// error: cannot retrieve the device path
				//  (more)   nothing
				//  (return) ignored
				noHIDPath = 0x80000E00,

				// error: cannot open the HID for reading
				//  (more)   zero-terminated path (UTF-16)
				//  (return) ignored
				hidNotReadable,

				// error: cannot retrieve the HIDs input report protocol
				//  (more)   zero-terminated path (UTF-16)
				//  (return) ignored
				noReportProtocol,

				// error: cannot retrieve the HIDs capabilities
				//  (more)   zero-terminated path (UTF-16)
				//  (return) ignored
				noCapabilities,

				// error: cannot retrieve the HIDs buttons
				//  (more)   zero-terminated path (UTF-16)
				//  (return) ignored
				noButtons,

				// error: cannot retrieve the HIDs axes
				//  (more)   zero-terminated path (UTF-16)
				//  (return) ignored
				noAxes

			}; } // Error


		} // RawInput


	} // Win32


} // OS