// Fundamental data types & functions; compiler-independent syntax sugar. The fundamental stuff is mostly compiler-specific
//  macro magic and should be read bottom-up.



#if _M_AMD64 || __x86_64__ // Visual C++ || Clang/GCC
#	define COMPILED_FOR_X86_64 1
#	define COMPILED_FOR_LITTLE_ENDIAN 1
#elif _M_IX86 || __i386__ // Visual C++ || Clang/GCC
#	define COMPILED_FOR_X86_32 1
#	define COMPILED_FOR_LITTLE_ENDIAN 1
#else
#	error this code is for x86-32 and x86-64
#endif

#if 0 == TARGET_OS_IS_WIN32 | TARGET_OS_IS_NT | TARGET_OS_IS_UNIX // Auto-detect target OS?
#	if _WIN32
#		define TARGET_OS_IS_WIN32 1 // default to Win32
#		ifndef TARGET_OS_VERSION
#			define TARGET_OS_VERSION TARGET_OS_VERSION_WINDOWS_7
#		endif
#	else
#		error this code is for Win32 or NT
#	endif
#endif

// Test Clang first (it may emulate GCC or Visual C++). Clang and GCC are mostly compatible (with very few exceptions).
#if __clang__ || __GNUC__

#	if __clang__
#		define COMPILED_BY_CLANG 1
#		define IMPL_IMPORT_FROM(LIBRARY) __pragma(comment(lib, STRINGIZE(LIBRARY.lib))) extern "C"
#	else
#		define COMPILED_BY_GCC 1
#		define IMPL_IMPORT_FROM(LIBRARY) extern "C"
#	endif
#	define IMPL_IMPORTED __attribute__((visibility("default")))
#	define IMPL_EXPORTED __attribute__((visibility("default")))

#	define RESTRICT // Not yet supported? :(

#	define IMPL_INVOKE_DEBUGGER __builtin_trap()

#	if _DEBUG
#		define IMPL_UNREACHABLE (__builtin_trap(), __builtin_unreachable())
#	else
#		define IMPL_UNREACHABLE __builtin_unreachable()
#		define IMPL_MUST(CONDITION) ((CONDITION) || (__builtin_unreachable, 0))
#	endif

#	define IMPL_ABSTRACT_BASE // no optimizations known for Clang/GCC

#	define IMPL_INLINE __attribute__((always_inline))
#	define IMPL_OUTLINE __attribute__((noinline))
#	define IMPL_WONT_RETURN __attribute__((noreturn)) void

	typedef float IMPL_FLOATX4 __attribute__((vector_size(16)));
	typedef int IMPL_INTX4 __attribute__((vector_size(16)));
	typedef double IMPL_DOUBLEX2 __attribute__((vector_size(16)));

#	define IMPL_TODO(TEXT) _Pragma(STRINGIZE(GCC warning "TODO: " #TEXT))

#elif _MSC_VER >= 1900 // Microsoft Visual C++ 2015 or later?

#	define COMPILED_BY_VISUAL_CPP 1
#	if _WINDLL
#		define COMPILED_TO_DYNAMIC_LIBRARY 1
#	else
#		define COMPILED_TO_EXECUTABLE 1
#	endif

	// Disable warning
	//  C4127: conditional expression is constant  annoying with static ifs generated by templates
	//  C4200: nonstandard extension used : zero-sized array in struct/union  legal in C11
	//  C4201: nonstandard extension used: nameless struct/union  all major compilers support that
	//  C4324: structure was padded due to alignment specifier  thats what the specifier is for
	//  C4373: previous versions of the compiler did not override when parameters only differed by const/volatile qualifiers  M(
	//  C4624: destructor was implicitly defined as deleted  annoying with interfaces
	//  C4815: zero-sized array in stack object will have no elements  legal and expected
#	pragma warning(disable: 4127 4200 4201 4324 4373 4624 4815)

	// These are built-in types, but theyre only visible when defined.
	__if_not_exists(__m128) { // avoid collisions with Microsoft headers
		 union __declspec(intrin_type align(16)) __m128  {  float f32[4]; };
		 union __declspec(intrin_type align(16)) __m128i {    int i32[4]; };
		struct __declspec(intrin_type align(16)) __m128d { double f64[2]; };
	}
#	define IMPL_FLOATX4  __m128
#	define IMPL_INTX4    __m128i
#	define IMPL_DOUBLEX2 __m128d

#	define RESTRICT __restrict
#	ifndef _CPPUNWIND
#		define noexcept // prevent warning C4577 (using noexcept although exceptions are disabled)
#	endif

#	define IMPL_INVOKE_DEBUGGER __debugbreak()

#	if _DEBUG
#		define IMPL_UNREACHABLE (__debugbreak(), __assume(0))
#	else
#		define IMPL_UNREACHABLE __assume(0)
#		define IMPL_MUST(CONDITION) __assume(bool(CONDITION))
#	endif

#	define IMPL_ABSTRACT_BASE __declspec(novtable)

#	define IMPL_INLINE __forceinline
#	define IMPL_OUTLINE __declspec(noinline)
#	define IMPL_WONT_RETURN void __declspec(noreturn)

#	define IMPL_TODO_QUOTE(TEXT) STRINGIZE(TEXT)
#	define IMPL_TODO(TEXT) __pragma(message(__FILE__ " (" IMPL_TODO_QUOTE(__LINE__) "): TODO: " TEXT))
#	define IMPL_IMPORT_FROM(LIBRARY) __pragma(comment(lib, STRINGIZE(LIBRARY.lib))) extern "C"
#	define IMPL_EXPORTED __declspec(dllexport)
#	define IMPL_IMPORTED __declspec(dllimport)

#else
#	error this code requires Visual C++ 2015, Clang, or GCC
#endif

// Prints the given string in a printf()-like manner to the debugger (in debug as well as in release builds).
//  Always consider using DEBUG_OUTPUT() instead.
void IMPL_OUTLINE DIAGNOSTIC_OUTPUT(char const * toZeroTerminatedFormat, ...);

#if _DEBUG
	bool shouldIgnoreAssertion(char const *);
	// The program is interrupted if the given condition fails.
#	define MUST(condition) void((condition) || shouldIgnoreAssertion(#condition) || (INVOKE_DEBUGGER, true))

#	define DEBUG_OUTPUT DIAGNOSTIC_OUTPUT
#else
	// The state information is used as hints to the optimizer. This allows optimization beyond traditional program analysis.
#	define MUST(condition) IMPL_MUST(condition)

#	define DEBUG_OUTPUT(ZERO_TERMINATED_FORMAT, ...)
#endif

// Compiling is interrupted if the given condition fails.
#define MUST_STATIC(CONDITION) static_assert(CONDITION, #CONDITION)

#if TARGET_OS_IS_WIN32 || TARGET_OS_IS_NT

#	define TARGET_OS_VERSION_WINDOWS_2000  50
#	define TARGET_OS_VERSION_WINDOWS_XP    51
#	define TARGET_OS_VERSION_WINDOWS_VISTA 60
#	define TARGET_OS_VERSION_WINDOWS_7     61
#	define TARGET_OS_VERSION_WINDOWS_8     62
#	define TARGET_OS_VERSION_WINDOWS_8_1   63
#	define TARGET_OS_VERSION_WINDOWS_10   100

	// Windows native character encoding is UTF-16. See MSDN, Unicode:
	//    [] functions use UTF-16 (wide character) encoding, which is [] the one used for native Unicode encoding on Windows
	//     operating systems.
#	define IMPL_NATIVEUNIT UTF16Unit
#	define IMPL_N(LITERAL) UTF16(LITERAL)

	// Win32/NT calls: stdcall; C run-time: cdecl.
#	if COMPILED_BY_VISUAL_CPP
#		define IMPL_API_CALL __stdcall
#		define IMPL_C_CALL __cdecl
#	elif COMPILED_BY_CLANG || COMPILED_BY_GCC
#		define IMPL_API_CALL __attribute__((stdcall))
#		define IMPL_C_CALL __attribute__((cdecl))
#	else
#		error TODO
#	endif

#elif TARGET_OS_IS_UNIX

#	define IMPL_NATIVEUNIT UTF8Unit
#	define IMPL_N(LITERAL) UTF8(LITERAL)

	// Useless: there is only one calling convention:
#	define IMPL_API_CALL
#	define IMPL_C_CALL

#else
#	error TODO
#endif

// Informs the linker of the given library and suppresses name mangling for following functions and structures.
#define import_from(LIBRARY) IMPL_IMPORT_FROM(LIBRARY)
// The object is imported from an external module without name mangling.
#define IMPORTED extern "C" IMPL_IMPORTED
// The object is exported to an external module without name mangling.
#define EXPORTED extern "C" IMPL_EXPORTED
// The calling convention which is native to the operating system.
#define API_CALL IMPL_API_CALL
// The calling convention which is native to the C run-time library.
#define C_CALL IMPL_C_CALL

using Byte = unsigned char;
// A size of something, in bytes (unsigned); large enough to address any byte in the address space (std::uintptr_t). Recycle
//  size_t, which is the unsigned integer type of the result of the sizeof operator.
using USize = decltype(sizeof 0);
// A size of something, in bytes (signed); large enough to store any size difference in the address space (std::ptrdiff_t).
//    If two pointers point to the same object or both point one past the end of the same array or both are null, and the two
//     pointers are subtracted, the result compares equal to the value 0 converted to the type std::ptrdiff_t.
using SSize = decltype((int *)0 - (int *)0);

// A number of some things (unsigned); large enough for any amount that fits the address space.
using UCount = USize;
// A number of some things (signed) or a difference in amounts; large enough for any amount that fits the address space.
using SCount = SSize;

using UInt1B = unsigned char;
using SInt1B = signed char; // signed is explicit for char
using UInt2B = unsigned short;
using SInt2B = short;
using UInt4B = unsigned;
using SInt4B = int;
using UInt8B = unsigned long long;
using SInt8B = long long;

// Integer types with a minimum size and an actual size that is most efficient on the target architecture:
#if COMPILED_FOR_X86_32 || COMPILED_FOR_X86_64 // 32-bit on x86-64 is just as fast, but with shorter opcodes
	using UIntLeast1B = UInt4B;
	using SIntLeast1B = SInt4B;
	using UIntLeast2B = UInt4B;
	using SIntLeast2B = SInt4B;
	using UIntLeast4B = UInt4B;
	using SIntLeast4B = SInt4B;
	using UIntLeast8B = UInt8B;
	using SIntLeast8B = SInt8B;
#else
#	error TODO
#endif

using UInt2B_LE = UInt2B; // 2-B unsigned int, Little Endian
using UInt4B_LE = UInt4B; // 4-B unsigned int, Little Endian
using UInt8B_LE = UInt8B; // 8-B unsigned int, Little Endian
using SInt2B_LE = SInt2B; // 2-B signed int, Little Endian
using SInt4B_LE = SInt4B; // 4-B signed int, Little Endian
using SInt8B_LE = SInt8B; // 8-B signed int, Little Endian

using UInt2B_BE = UInt2B; // 2-B unsigned int, Big Endian
using UInt4B_BE = UInt4B; // 4-B unsigned int, Big Endian
using UInt8B_BE = UInt8B; // 8-B unsigned int, Big Endian
using SInt2B_BE = SInt2B; // 2-B signed int, Big Endian
using SInt4B_BE = SInt4B; // 4-B signed int, Big Endian
using SInt8B_BE = SInt8B; // 8-B signed int, Big Endian

// Explicit padding for data structures.
//   alignment is useless legacy on x86; using byte packing instead
//     no modern x86 CPU is slow on unaligned access
//     not even with SSE/AVX
//     atomic operations and gather/scatter are the only exceptions
//     cache locality is more important
//   3rd-party structures need explicit padding to remain compatible
//   cannot use a named variable for that
//     breaks initializer lists
//     confusing with 3rd-party documentation
//   anonymous bit fields are the perfect solution
#define PAD_1B UInt1B : 8
#define PAD_2B UInt2B : 16
#define PAD_3B UInt1B : 8; UInt2B : 16
#define PAD_4B UInt4B : 32
#if COMPILED_FOR_X86_32
#	define PAD_4B_ON_X86_64
#elif COMPILED_FOR_X86_64
#	define PAD_4B_ON_X86_64 PAD_4B
#else
#	error TODO
#endif

// NaN and infinities for floating-point numbers: There is very limited compiler support for these (e.g. Visual C++ wont
//  optimize anything thats not written out), so we still need macros for that.
#if COMPILED_BY_VISUAL_CPP
#	pragma warning(disable: 4056 4756) // NaN and INF are created using a controlled overrun, so disable the according warning
#endif
#define NAN_FLOAT4B (3.4028234e38f * 2.0f * 0.0f)
#define NAN_FLOAT8B (1.7976931348623157e308 * 2.0 * 0.0)
#define INF_FLOAT4B (3.4028234e38f * 2.0f) // positive infinite 32-bit floating-point number
#define INF_FLOAT8B (1.7976931348623157e308 * 2.0) // positive infinite 64-bit floating-point number
#define PI_FLOAT4B  3.14159265f
#define PI_FLOAT8B  3.141592653589793

// A 7-bit character.
using ASCIICharacter = char;
// A 8-bit code unit of a Unicode code point. C++14 defines char to be suitable for UTF-8.
using UTF8Unit = unsigned char;
// A 16-bit code unit of a Unicode code point.
using UTF16Unit = char16_t;
// A code unit of the encoding which is native to the operating system.
using NativeCodeUnit = IMPL_NATIVEUNIT;

// A string literal in UTF-8.
#define UTF8(LITERAL) IMPL_CONCATENATE(u8, LITERAL)
// A string literal in UTF-16.
#define UTF16(LITERAL) IMPL_CONCATENATE(u, LITERAL)
// A string literal in the character encoding native to the operating system.
#define N(LITERAL) IMPL_N(LITERAL)

#define local static // the function or variable is invisible to other source files
#define locals namespace // the functions, variables, and types are invisible to other source files
#define import extern "C" // functions or structures from external libraries
#define cycle for(;;) // infinite loop
#define no false
#define yes true
#define until(...) while(!(__VA_ARGS__))
#define UNUSED(NAME) (void)&NAME; // suppresses unreferenced local variable and unreferenced formal parameter warnings
#define uses // a forward declaration  this just makes the code easier to understand
#define STRINGIZE(TEXT) #TEXT // puts the parameter in double quotes
#define IMPL_CONCATENATE(A, B) A##B

// Contrasting C++ inline keyword, which controls linkage, INLINE_CODE asks for inlining of the generated assembly. Handy
//  for functions with just one caller, as Visual C++ does no single call site inlining.
#define INLINE_CODE IMPL_INLINE
// Prevents the compiler from inlining the assembly generated for the function (handy to suppress bloat).
#define OUTLINE_CODE IMPL_OUTLINE
// The function raises an exception, or terminates the thread, . Avoids useless warnings and improves code generation.
#define WONT_RETURN IMPL_WONT_RETURN
// Breaks to the debugger (or crashes the program if none is attached).
#define INVOKE_DEBUGGER IMPL_INVOKE_DEBUGGER
// Guarantees to the compiler that a specific branch cannot be reached. Suppresses false warnings and optimizes code generation.
#define UNREACHABLE IMPL_UNREACHABLE

// Guarantees that a class is never instantiated but instead serves as a pure interface. Improves optimization of virtual
//  function tables.
#define ABSTRACT_BASE IMPL_ABSTRACT_BASE

#define TODO(TEXT) IMPL_TODO(TEXT)

template <class T, UCount S> static char (&implCapacityOf(T (* const)[S]))[S];
template <class C, class T, UCount S> static char (&implCapacityOf(T (C::*)[S]))[S];
#define CAPACITY_OF(ARRAY) (sizeof implCapacityOf(&(ARRAY))) // the number of elements in the given array

void * operator new(USize, void *) noexcept;
void operator delete(void *, void *) noexcept;
void * operator new [](USize, void *) noexcept;
void operator delete [](void *, void *) noexcept;


// Returns an iterator to the beginning (the first element) of the given array.
template <typename T, UCount length> T * toBeginningOf(T (&array)[length]) {
	return array;
}

// Returns an iterator to the end (past the last element) of the given array.
template <typename T, UCount length> T * toEndOf(T (&staticArray)[length]) {
	return staticArray + length;
}

// Returns the capacity of the given array.
template <typename T, UCount length> UCount capacityOf(T (&)[length]) {
	return length;
}

// Returns the size, in bytes, of the given array.
template <typename T, UCount length> USize sizeOf(T (&)[length]) {
	return length * sizeof(T);
}

// Shifts the given pointer by the given amount of bytes.
// Using byte offsets instead of indices speeds up execution considerably (especially on weak platforms like Intel Atom).
template <typename T> T * byteOffset(
	T *   pointer,
	SSize offsetInBytes
) {
	return (T *)(((Byte *)pointer) + offsetInBytes);
}

// Returns the distance from a to b, in bytes.
template <typename T> SSize byteDistance(T * a, T * b) {
	// Works around poor pointer math optimization in Visual C++. Assume a nave implementation:
	//
	//    struct T; // size: 113 B
	//    T * a;
	//    T * b;
	//    return sizeof(T) * (b - a);
	//
	//  On CPU level, a and b are byte addresses, but C++ pointer math requires b - a to return the difference in number of
	//   elements. Therefore, the following instruction sequence is generated:
	//
	//    x := b - a    # 1 cycle
	//    x := x / 113  # 30 cycles
	//    x := x * 113  # 3 cycles
	//
	//  /113*113 seems redundant, but in integer math, X / Y * Y == X holds true ONLY if X is a multiple of Y. This is clearly the
	//  case, as a and b point to the same array  but this information is lost in compilers. This nave implementation takes
	//  34 cycles to execute.
	// Manually casting to byte pointers solves these problems:
	//
	//    x := b - a  # 1 cycle
	//
	return ((Byte *)b) - ((Byte *)a);
}


// Zero or more elements of the same type.
template <typename Element> struct Range {
	Element * toBeginning;
	Element * toEnd;

	Element & operator [](UCount index) const {
		MUST(toEnd >= toBeginning);
		MUST(index < UCount(toEnd - toBeginning));
		return toBeginning[index];
	}

	// (Implicitly) converts the range to a read-only range.
	operator Range<Element const>() const {
		return { toBeginning, toEnd };
	}

	// Any elements in the range?
	operator bool() const {
		return toEnd > toBeginning;
	}

};

// Returns an iterator to the beginning (the first element) of the given range.
template <typename T> T * toBeginningOf(Range<T> const & range) {
	MUST(range.toBeginning <= range.toEnd);
	return range.toBeginning;
}

// Returns an iterator to the end (past the last element) of the given range.
template <typename T> T * toEndOf(Range<T> const & range) {
	MUST(range.toBeginning <= range.toEnd);
	return range.toEnd;
}

// Returns the number of elements in the given range.
template <typename T> UCount lengthOf(Range<T> const & range) {
	MUST(range.toBeginning <= range.toEnd);
	return range.toEnd - range.toBeginning;
}

// Compares the number of elements in the given range to the given number.
// Necessary because Visual C++ treats pointers as integers, and employs a full integer division when subtracting pointers.
template <typename T> bool lengthEquals(
	Range<T> const & range,
	UCount           length
) {
	MUST(range.toEnd >= range.toBeginning);
	// optimize
	//    n == ((Byte*)end - (Byte*)beginning) / sizeof(T)
	//  to
	//    n * sizeof(T) == ((Byte*)end - (Byte*)beginning)
	return length * sizeof(T) == USize(byteDistance(range.toBeginning, range.toEnd));
}

// Compares the number of elements in the given range to the given number.
// Necessary because Visual C++ treats pointers as integers, and employs a full integer division when subtracting pointers.
template <typename T> bool isIndexInBounds(
	T const * toBeginning,
	UCount    index,
	T const * toEnd
) {
	MUST(toEnd >= toBeginning);
	// optimize
	//    n < ((Byte*)end - (Byte*)beginning) / sizeof(T)
	//  to
	//    n * sizeof(T) < ((Byte*)end - (Byte*)beginning)
	return index * sizeof(T) < USize(byteDistance(toBeginning, toEnd));
}

// Returns the size, in bytes, of the given range.
template <typename T> USize sizeOf(Range<T> const & range) {
	MUST(range.toEnd >= range.toBeginning);
	return byteDistance(range.toBeginning, range.toEnd);
}

// Determines whether the given range covers no elements.
template <typename T> bool isEmpty(Range<T> const & range) {
	MUST(range.toEnd >= range.toBeginning);
	return range.toEnd == range.toBeginning;
}

// Creates a range from the given elements.
template <typename T> Range<T> range(
	T * toItsBeginning,
	T * toItsEnd
) {
	MUST(toItsEnd >= toItsBeginning);
	return { toItsBeginning, toItsEnd };
}
template <typename T> Range<T> range(T * toItsBeginning, UCount itsLength) {
	return { toItsBeginning, toItsBeginning + itsLength };
}
template <typename T, UCount length> Range<T> range(T (&array)[length]) {
	return { array, array + length };
}

// Casts the given byte range to the given other element type.
template <
	typename NewT,
	typename OldT
> Range<NewT> const reinterpreted(
	Range<OldT> const & bytes
) {
	MUST(0 == sizeOf(bytes) % sizeof(NewT)); // size must match new type
	return {
		reinterpret_cast<NewT *>(bytes.toBeginning),
		reinterpret_cast<NewT *>(bytes.toEnd)
	};
}

template <typename Structure> Structure const & read(Range<Byte const> & bytes) {
	MUST(sizeof(Structure) <= sizeOf(bytes));
	auto & result = *reinterpret_cast<Structure const *>(bytes.toBeginning);
	bytes.toBeginning += sizeof result;
	return result;
}

// Determines whether the given value is within the given lower and upper limit.
template <
	typename T
> bool isWithin(
	T const & lowerLimit,
	T const & value,
	T const & upperLimit
) {
	MUST(lowerLimit <= upperLimit);
	return lowerLimit <= value && value <= upperLimit;
}
bool isWithin(
	UInt1B lowerLimit,
	UInt1B value,
	UInt1B upperLimit
);
bool isWithin(
	UInt2B lowerLimit,
	UInt2B value,
	UInt2B upperLimit
);
bool isWithin(
	UInt4B lowerLimit,
	UInt4B value,
	UInt4B upperLimit
);
bool isWithin(
	UInt8B lowerLimit,
	UInt8B value,
	UInt8B upperLimit
);
bool isWithin(
	SInt1B lowerLimit,
	SInt1B value,
	SInt1B upperLimit
);
bool isWithin(
	SInt2B lowerLimit,
	SInt2B value,
	SInt2B upperLimit
);
bool isWithin(
	SInt4B lowerLimit,
	SInt4B value,
	SInt4B upperLimit
);
bool isWithin(
	SInt8B lowerLimit,
	SInt8B value,
	SInt8B upperLimit
);

// Returns the minimum of the given objects.
template <
	typename Object
> Object const & minimumOf(
	Object const & a,
	Object const & b
) {
	return (a < b) ? a : b;
}
float minimumOf(float, float);
float minimumOf(float, float, float);
double minimumOf(double, double);

// Returns the maximum of the given objects.
template <
	typename Object
> Object const & maximumOf(
	Object const & a,
	Object const & b
) {
	return (a < b) ? b : a;
}
float maximumOf(float, float);
float maximumOf(float, float, float);
double maximumOf(double, double);

// Clamps the given value to the given range.
UInt4B clamped(
	UInt4B lowerLimit,
	UInt4B value,
	UInt4B upperLimit
);
SInt4B clamped(
	SInt4B lowerLimit,
	SInt4B value,
	SInt4B upperLimit
);
UInt8B clamped(
	UInt8B lowerLimit,
	UInt8B value,
	UInt8B upperLimit
);
float clamped(
	float lowerLimit,
	float value,
	float upperLimit
);
double clamped(
	double lowerLimit,
	double value,
	double upperLimit
);

float absoluteOf(float);
double absoluteOf(double);
UInt1B absoluteOf(SInt1B);
UInt2B absoluteOf(SInt2B);
UInt4B absoluteOf(SInt4B);
UInt8B absoluteOf(SInt8B);

// Returns no for any kind of NaN, but yes for infinities.
bool isNumber(float);
bool isNumber(double);

// no for positive or negative infinity and any kind of NaN.
bool isFinite(float);
bool isFinite(double);

// yes for subnormal floating-point numbers but not for zero.
bool isSubnormal(float);

//  Are both operands positive?
//  Are both operands negative?
//  Is one or both operands zero (because zeros sign is pointless)?
bool sameSign(float, float);

float doubled(float);
double doubled(double);
SInt4B doubled(SInt4B);
UInt4B doubled(UInt4B);
UInt8B doubled(UInt8B);

// Rounds to integer.
//   must be in [-2 147 483 648, +2 147 483 520] range
//     deviation from signed 4-B integer minimum/maximum is due to floating-point accuracy
SInt4B rounded(float);

// Rounds the given IEEE754 double-precision floating-point number to a four-byte signed integer. Over- and underruns are
//  undefined.
SInt4B rounded(double);
// Rounds the given IEEE754 single-precision floating-point number to an 8-B signed integer. Over- and underruns are undefined.
SInt8B roundedTo8B(float);
// Rounds the given IEEE754 double-precision floating-point number to an 8-B signed integer. Over- and underruns are undefined.
SInt8B roundedTo8B(double);

// Rounds the given fixed-point number with the given number of fraction bits to the nearest integer.
SInt8B roundedFixedPoint(
	SInt8B number,
	UInt1B numberOfFractionBits
);
SInt4B roundedFixedPoint(
	SInt4B number,
	UInt1B numberOfFractionBits
);

// Converts to integer by truncating all digits after the decimal point.
//   must be in [-2 147 483 648, +2 147 483 520] range
//     deviation from signed 4-B integer maximum is due to floating-point accuracy
//   other interpretations:
//     floored() for positive numbers; ceiled() for negative numbers
//     rounding towards zero
SInt4B truncated(float);

UInt1B flippedEndianness(UInt1B);
UInt2B flippedEndianness(UInt2B);
UInt4B flippedEndianness(UInt4B);
UInt8B flippedEndianness(UInt8B);

// Interprets the given value as Big Endian. If the current architecture is Little Endian, bytes are returned swapped.
UInt1B bigEndian(UInt1B);
UInt2B bigEndian(UInt2B);
UInt4B bigEndian(UInt4B);
UInt8B bigEndian(UInt8B);
SInt1B bigEndian(SInt1B);
SInt2B bigEndian(SInt2B);
SInt4B bigEndian(SInt4B);
SInt8B bigEndian(SInt8B);

// Interprets the given value as Little Endian. If the current architecture is Big Endian, bytes are returned swapped.
UInt1B littleEndian(UInt1B);
UInt2B littleEndian(UInt2B);
UInt4B littleEndian(UInt4B);
UInt8B littleEndian(UInt8B);
SInt1B littleEndian(SInt1B);
SInt2B littleEndian(SInt2B);
SInt4B littleEndian(SInt4B);
SInt8B littleEndian(SInt8B);

// Shifts an integer n bits to the right, padding with the sign.
//   use for clarity (C/C++ do not define whether the >> operator pads in zeroes or the sign bit)
//   shift count must be below number of bits in the operand (C/C++ convention)
//   equals division by a power of two, rounding towards negative infinity
SIntLeast1B shiftRightArithmetic(SInt1B, UIntLeast1B);
SIntLeast2B shiftRightArithmetic(SInt2B, UIntLeast1B);
SIntLeast4B shiftRightArithmetic(SInt4B, UIntLeast1B);
SIntLeast8B shiftRightArithmetic(SInt8B, UIntLeast1B);

UInt4B rightRotated(UInt4B, UIntLeast1B bits);


// Returns the two given ASCII characters as an integer as if loaded from memory.
// Handy for magic numbers in file formats (e.g. BM in .bmp), where two comparisons of two characters each can be replaced
//  with one comparison of two-byte numbers.
UInt2B asInt(ASCIICharacter const (& twoCharacters)[3]);
// Returns the four given ASCII characters as an integer as if loaded from memory (FourCC).
// Handy for magic numbers in file formats (e.g. RIFF in .avi and .wav), where four comparisons of two characters each can be
//  replaced with one comparison of four-byte numbers.
UInt4B asInt(ASCIICharacter const (& fourCharacters)[5]);
// Returns the eight given ASCII characters as an integer as if loaded from memory.
// Handy for magic numbers in file formats, where eight comparisons of two characters each can be replaced with one comparison
//  of eight-byte numbers.
UInt8B asInt(ASCIICharacter const (& eightCharacters)[9]);

// Returns the floored binary logarithm of the given number (equals the index of the most significant bit set, with 0 for LSB).
//   undefined for zero
UIntLeast1B binaryLogarithmOf(USize);

// Determines whether the given integer is evenly divisable by two.
//   zero is even
bool isEven(SInt4B);

// Determines whether the given number is negative.
//   negative zero returns yes
//   NaN is undefined
bool isNegative(float);



// Base class for opque objects. A pointer to such an object is a handle.
//   cannot be instantiated (typically retrieved from external code)
//   cannot be copied
//   cannot be destroyed
struct Opaque {
	Opaque() = delete;
	Opaque(Opaque const &) = delete;
	~Opaque() = delete;
};


__if_not_exists(UUID) { // avoid collisions with Microsoft headers
	struct UUID {
		UInt4B Data1;
		UInt2B Data2;
		UInt2B Data3;
		UInt1B Data4[8];
	};

	bool operator ==(UUID const &, UUID const &);
	bool operator !=(UUID const &, UUID const &);
}



// The error handling interface.
//   typically the programs UI
//   offers
//     localization
//       no strings in algorithms; its all the UIs job
//       output can be customized and localized at will: language-independent hexadecimal error codes for simple apps; detailed
//        localized error messages for rich applications
//     progress bars
//       for long-running tasks
//       no multi-threading required (simple to use)
//     cancellation
//       crucial for UI, optional for simple apps
//     resumable errors
//       Is it a warning or an error? The UI decides
//       UI decides whether error handling should be strict or fault-tolerant
//     callback-like behaviour
//       file not found: Allow user to insert CD-ROM?
//   can be translated to any language (successfully handled C++ errors in Python & C#)
struct Supervisor {

	// Implementation-defined context.
	//   always passed as first parameter to onEvent()
	//   commonly used for this pointer
	void * context;

	// Implementation-defined event callback.
	//   must not be nullptr
	//   must not raise exceptions
	//   should return 0 on unknown events
	SSize (API_CALL * onEvent)(
		void *                 context,
		UInt4B                 event,
		void const *           more,
		NativeCodeUnit const * callstackOrNull
	);

};

// Reports the given event to the given supervisor (if existent).
// Designed for inlining & quick rejection by the optimizer if the supervisor is nullptr.
SSize report(
	Supervisor const * supervisorOrNull,
	UIntLeast4B        event,
	void const *       more
);
SSize report(
	Supervisor const * supervisorOrNull,
	UIntLeast4B        event,
	UCount             more
);
SSize report(
	Supervisor const * supervisorOrNull,
	UIntLeast4B        event
);

SSize OUTLINE_CODE report(
	Supervisor const & ,
	UIntLeast4B        event,
	void const *       more
);
SSize OUTLINE_CODE report(
	Supervisor const & ,
	UIntLeast4B        event
);

// Reports an Info::progress event to the given supervisor (if existent).
bool onProgress(
	Supervisor const * supervisorOrNull,
	UIntLeast4B        perMille
);



// Error code layout:
//    0000 0000  0FFF FFFF: success, progress, internal
//    1000 0000  3FFF FFFF: information
//    4000 0000  7FFF FFFF: warnings
//    8000 0000  FFFF FFFF: errors

namespace Info { enum : UInt4B {

	// The operation completed successfully. Not an event.
	ok = 0,

	// The operation has not yet completed (initial state of all asynchronous operations).
	//  (more)   nothing
	//  (return) ignored
	pending,

	// There is nothing to do. This may indicate a failure on higher levels.
	//  (more)   nothing
	//  (return) ignored
	idle,

	// The operation progressed. There is a chance to cancel it; otherwise, the UI should update progress bars.
	//  (more)   current progress, in per mille (percent does not suffice for progress bars on widescreen monitors)
	//             in the range of [0, 1000]
	//             never decreasing
	//             not necessarily increasing (i.e. you may get the same value many times in a row)
	//  (return) zero to cancel the operation; any other value to keep going
	//             ignored if the operation cannot be cancelled
	//             cancellation may take a while
	progress,

	// Any kind of information as an ANSI string (common for 3rd-party code with hard-coded strings).
	//  (more)   pointer to zero-terminated ANSI string
	//  (return) ignored
	unspecifiedANSI,
	unspecifiedUTF8,
	unspecifiedUTF16

}; } // Info

namespace Warning { enum : UInt4B {

	// Something went wrong and only translates into an ANSI string (common for 3rd-party code with hard-coded warning strings).
	//  (more)   pointer to zero-terminated ANSI string
	//  (return) yes to ignore; no to abort
	unspecifiedANSI = 0x40000000,
	unspecifiedUTF8,
	unspecifiedUTF16

}; } // Warning

namespace Error { enum : UInt4B {

	// Something went wrong.
	//  (more)   nothing
	//  (return) ignored
	error = 0x80000000,

	// The operation has been cancelled by the user.
	//  (more)   nothing
	//  (return) ignored
	aborted,

	// Something went wrong and only translates into an ANSI string (common for 3rd-party code with hard-coded error strings).
	//  (more)   pointer to zero-terminated ANSI string
	//  (return) ignored
	unspecifiedANSI,
	unspecifiedUTF8,
	unspecifiedUTF16,

	// Virtual memory exhaustion.
	//  (more)   nothing
	//  (return) ignored
	outOfVirtualMemory,

	// GPU memory exhaustion.
	//  (more)   nothing
	//  (return) ignored
	outOfGPUMemory,

	fileNotFound,

	// A required EXE or DLL is missing.
	//  (more)   pointer to zero-terminated DLL path (UTF-16)
	//  (return) ignored
	missingModule,

	// A required EXE or DLL does not behave as expected. This is probably the result of someone messing with program files.
	//  (more)   pointer to zero-terminated DLL path (UTF-16)
	//  (return) ignored
	damagedModule

}; } // Error




// API namespaces from the framework are declared here. This is more convenient for parsers like IntelliSense and Doxygen.

// PCM formats, signal processing, etc.
namespace Audio { }

// Anything concerning file formats.
namespace Formats { }

// Color space transformations, pixel data structures, etc.
namespace GFX { }

// Platform-independent allocators and memory management.
namespace Heap { }

// International Business Machines, the US corporation that helped shaping the Personal Computer.
namespace IBM { }

// Geometry, algebra, trigonometry, etc.
namespace Math { }

// Operating system-dependent functionality.
namespace OS {

	// Native API of WindowsNT. Serves drivers and subsystems like Win32, OS/2, POSIX, and DOS.
	namespace NT { }

	// The Win32 API, which is supported by Windows95/98/ME and all NT-derived Windows versions.
	namespace Win32 {

		// The user interface of Windows9x, NT, 2000, XP, Vista, and 7: windows, dialogs, and menus. Applies to the classic shell
		//  as well as to Luna and Aeroglass.
		namespace ClassicUI { }

		// The Component Object Model is the fundamental Windows API for language-independent component development. Many APIs,
		//  including the Shell and WinRT, are built on top of it.
		namespace COM { }

		// Windows cryptography API: Hashes, signatures, encryption.
		namespace Crypto { }

		// Direct3D is the Windows API for 3D graphics hardware access. Version 9 from the June 2010 DirectX SDK is the last version
		//  to support WindowsXP.
		namespace D3D9 { }

		// Direct3D is the Windows API for 3D graphics hardware access. Version 11.1 is the last version to support Windows7.
		namespace D3D11 { }

		// The Debug Help API allows to save call stacks, gather symbol names, etc.
		namespace DbgHelp { }

		// The Graphics Device Interface can hardly be avoided in Win32 programming since the Classic UI builds upon it in so many
		//  places (e.g. drawing text to windows).
		namespace GDI { }

		// The WinINet API offers access to FTP and HTTP functionality. It is the Windows standard API for web browsers. 
		namespace INet { }

		// GDI+ was introduced as a successor to GDI. It is mostly forgotten today, but it offers a reliable image file decoder which
		//  is present on any Windows machine since XP. This makes it perfect for loading image files without distributing a library.
		namespace GDIPlus { }

		// RawInput is the Windows API for low-level input processing and Human Interface Device access. Windows messages and
		//  DirectInput are built on top of it.
		namespace RawInput { }

		namespace Registry { }

		// Whatever concerns the Explorer, Desktop, Taskbar, file associations and thumbnails, 
		namespace Shell { }

		// XAudio is the Windows API for game-oriented audio rendering. Version 2.7 from the June 2010 DirectX SDK is the last version
		//  to support WindowsXP.
		namespace XAudio2 { }

	}

}

// Random number generators.
namespace Random { }

// Parsing, lexing, code pages, etc.
namespace Text {

	// Functions for parsing C source code.
	// Not just handy for compilers: Many protocols use C conventions for literals (integers, floating-point numbers, strings).
	namespace C { }

}

// Human Interface Devices from the USB standard.
namespace USBHID { }