#include "framework.hpp"
#include "output streams.hpp"
#include "OS Win32.hpp"
#include "OS Win32 File.hpp"
#include "Text.hpp"



OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(ASCIICharacter const * string) const {
	// Find the strings end (write the loop because Visual C++ 2015 wont optimize string + strlen(string)):
	auto toEnd = string;
	until(0 == *toEnd) {
		++toEnd;
	}
	(*toWriter)(*this, string, toEnd);
	return *this;
}
ASCIIFormat const & ASCIIFormat::operator <<(Range<ASCIICharacter const> const & string) const {
	(*toWriter)(*this, string.toBeginning, string.toEnd);
	return *this;
}
ASCIIFormat const & ASCIIFormat::operator <<(ASCIICharacter character) const {
	(*toWriter)(*this, &character, &character + 1);
	return *this;
}

OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(UTF16Unit const * string) const {
	until(0 == *string) {
		auto output = (128 > *string) ? ASCIICharacter(*string) : ASCIICharacter('?');
		(*toWriter)(*this, &output, &output + 1);
		++string;
	}
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(Range<UTF16Unit const> const & string) const {
	auto const * toUnit = string.toBeginning;
	while(toUnit < string.toEnd) {
		auto output = (128 > *toUnit) ? ASCIICharacter(*toUnit) : ASCIICharacter('?');
		(*toWriter)(*this, &output, &output + 1);
		++toUnit;
	}
	return *this;
}

OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(UInt1B number) const {
	ASCIICharacter result[sizeof "255"];
	(*toWriter)(*this, result, Text::toDecimalString(result, number));
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(UInt2B number) const {
	ASCIICharacter result[sizeof "65536"];
	(*toWriter)(*this, result, Text::toDecimalString(result, number));
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(UInt4B number) const {
	ASCIICharacter result[sizeof "4294967295"];
	(*toWriter)(*this, result, Text::toDecimalString(result, number));
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(UInt8B number) const {
	ASCIICharacter result[sizeof "18446744073709551615"];
	(*toWriter)(*this, result, Text::toDecimalString(result, number));
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(SInt1B number) const {
	ASCIICharacter result[sizeof "-128"];
	(*toWriter)(*this, result, Text::toDecimalString(result, number));
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(SInt2B number) const {
	ASCIICharacter result[sizeof "-32768"];
	(*toWriter)(*this, result, Text::toDecimalString(result, number));
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(SInt4B number) const {
	ASCIICharacter result[sizeof "-2147483648"];
	(*toWriter)(*this, result, Text::toDecimalString(result, number));
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(SInt8B number) const {
	ASCIICharacter result[sizeof "-9223372036854775808"];
	(*toWriter)(*this, result, Text::toDecimalString(result, number));
	return *this;
}
OUTLINE_CODE ASCIIFormat const & ASCIIFormat::operator <<(double number) const {
	ASCIICharacter result[349];
	(*toWriter)(*this, result, Text::toDecimalString(number, 17, '.', no, result));
	return *this;
}


// Implements writing from an ASCII output to an OS::Win32::File which is provided as context.
local void asciiFileWriter(
	ASCIIFormat const &    base,
	ASCIICharacter const * toBeginning,
	ASCIICharacter const * toEnd
) {
	MUST(toBeginning <= toEnd);
	auto & self = static_cast<ASCIIToFile const &>(base);
	if(nullptr != self.file) { // stdout may have been redirected to null
		WIN32_SHOULD_SUCCEED(((OS::Win32::File *)self.file)->tryToWrite(toBeginning, USize(toEnd - toBeginning), nullptr));
	}
}

ASCIIToFile::ASCIIToFile() {
	toWriter = &asciiFileWriter;
	file     = &OS::Win32::getStdout();
}

ASCIIToFile const out;